@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.81 1997/12/09 13:27:49 joda Exp $ */
+/* $Id: acconfig.h,v 1.90 1998/06/04 05:20:36 joda Exp $ */
 
 /* Define this to the type ssize_t should be */
 #undef ssize_t
@@ -76,6 +76,26 @@
 /* define if the system is missing a prototype for getusershell() */
 #undef NEED_GETUSERSHELL_PROTO
 
+/* define if prototype of gethostbyname is compatible with
+   `struct hostent *gethostbyname(const char *)
+   */
+#undef GETHOSTBYNAME_PROTO_COMPATIBLE
+
+/* define if prototype of gethostbyaddr is compatible with
+   `struct hostent *gethostbyaddr(const void *, size_t, int)
+   */
+#undef GETHOSTBYADDR_PROTO_COMPATIBLE
+
+/* define if prototype of getservbyname is compatible with
+   `struct servent *getservbyname(const char *, const char *)
+   */
+#undef GETSERVBYNAME_PROTO_COMPATIBLE
+
+/* define if prototype of openlog is compatible with
+   `void open(const char *, int, int)'
+   */
+#undef OPENLOG_PROTO_COMPATIBLE
+
 /* define if /bin/ls takes -A */
 #undef HAVE_LS_A
 
@@ -180,9 +200,7 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
  * Define NDBM if you are using the 4.3 ndbm library (which is part of
  * libc).  If not defined, 4.2 dbm will be assumed.
  */
-#if defined(HAVE_DBM_FIRSTKEY)
-#define NDBM
-#endif
+#undef NDBM
 
 /* ftp stuff -------------------------------------------------- */
 
@@ -247,6 +265,9 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 /* Define this if you have a working getmsg */
 #undef HAVE_GETMSG
 
+/* Define to enable old kdestroy behavior */
+#undef LEGACY_KDESTROY
+
 /* Define to enable new master key code */
 #undef RANDOM_MKEY
 
@@ -287,9 +308,11 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
  */
 #if _AIX
 #define _ALL_SOURCE
-#define _POSIX_SOURCE
-/* this is left for hysteric reasons :-) */
-#define unix /* well, ok... */
+/* XXX this is gross, but kills about a gazillion warnings */
+struct ether_addr;
+struct sockaddr;
+struct sockaddr_dl;
+struct sockaddr_in;
 #endif
 
 /*
@@ -320,4 +343,9 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 
 #ifdef __EMX__
 #define _EMX_TCPIP
+#define MAIL_USE_SYSTEM_LOCK
+#endif
+
+#ifdef ROKEN_RENAME
+#include "roken_rename.h"
 #endif
