@@ -9,20 +9,22 @@
 
 #include "adm_locl.h"
 
-RCSID("$Id: kdb_destroy.c,v 1.6 1996/03/25 13:11:03 bg Exp $");
+RCSID("$Id: kdb_destroy.c,v 1.8 1997/12/14 23:45:29 assar Exp $");
 
 int
-main(void)
+main(int argc, char **argv)
 {
     char    answer[10];		/* user input */
     char    dbm[256];		/* database path and name */
-    char    dbm1[256];		/* database path and name */
 #ifdef HAVE_NEW_DB
     char   *file;               /* database file names */
 #else
     char   *file1, *file2;	/* database file names */
+    char    dbm1[256];		/* database path and name */
 #endif
 
+    set_progname (argv[0]);
+
     strcpy(dbm, DBM_FILE);
 #ifdef HAVE_NEW_DB
     file = strcat(dbm, ".db");
@@ -44,13 +46,12 @@ main(void)
 	if (unlink(file1) == 0 && unlink(file2) == 0)
 #endif
 	  {
-	    fprintf(stderr, "Database deleted at %s\n", DBM_FILE);
+	    warnx ("Database deleted at %s", DBM_FILE);
 	    return 0;
 	  }
 	else
-	    fprintf(stderr, "Database cannot be deleted at %s\n",
-		    DBM_FILE);
+	    warn ("Database cannot be deleted at %s", DBM_FILE);
     } else
-	fprintf(stderr, "Database not deleted.\n");
+        warnx ("Database not deleted at %s", DBM_FILE);
     return 1;
 }
