@@ -38,7 +38,7 @@
 
 #include "config.h"
 
-RCSID("$Id: kstring2key.c,v 1.12 1997/04/01 08:17:13 joda Exp $");
+RCSID("$Id: kstring2key.c,v 1.13 1997/07/24 07:45:29 bg Exp $");
 
 #include <stdio.h>
 #include <string.h>
@@ -55,17 +55,31 @@ RCSID("$Id: kstring2key.c,v 1.12 1997/04/01 08:17:13 joda Exp $");
 static void
 usage(void)
 {
-    fprintf(stderr, "Usage: %s [-c AFS cellname] [ password ]\n",
+    fprintf(stderr,
+	    "Usage: %s [-c AFS cellname] [ -5 krb5salt ] [ password ]\n",
 	    __progname);
+    fprintf(stderr,
+	    "       krb5salt is realmname APPEND principal APPEND instance\n");
     exit(1);
 }
 
+static
+void
+krb5_string_to_key(char *str,
+		   char *salt,
+		   des_cblock *key)
+{
+    strcat(str, salt);
+    des_string_to_key(str, key);
+}
+
+
 int
 main(int argc, char **argv)
 {
     des_cblock key;
     char buf[1024];
-    char *cellname = 0;
+    char *cellname = 0, *salt = 0;
 
     set_progname (argv[0]);
 
@@ -75,7 +89,13 @@ main(int argc, char **argv)
 	    argv += 2;
 	    argc -= 2;
 	}
-    if (argc >= 2 && argv[1][0] == '-' && argv[1][1] == 'h')
+    else if (argc >= 3 && argv[1][0] == '-' && argv[1][1] == '5')
+	{
+	    salt = argv[2];
+	    argv += 2;
+	    argc -= 2;
+	}
+    if (argc >= 2 && argv[1][0] == '-')
 	usage();
 
     switch (argc) {
@@ -93,18 +113,25 @@ main(int argc, char **argv)
 
     if (cellname != 0)
 	afs_string_to_key(buf, cellname, &key);
+    else if (salt != 0)
+        krb5_string_to_key(buf, salt, &key);
     else
 	des_string_to_key(buf, &key);
 
     {
 	int j;
 	unsigned char *tkey = (unsigned char *) &key;
+	printf("ascii = ");
 	for(j = 0; j < 8; j++)
 	    if(tkey[j] != '\\' && isalpha(tkey[j]) != 0)
 		printf("%c", tkey[j]);
 	    else
 		printf("\\%03o",(unsigned char)tkey[j]);
 	printf("\n");
+	printf("hex   = ");
+	for(j = 0; j < 8; j++)
+	    printf("%02x",(unsigned char)tkey[j]);
+	printf("\n");
     }
     exit(0);
 }
