@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: bsd_locl.h,v 1.101 1997/12/04 14:19:30 assar Exp $ */
+/* $Id: bsd_locl.h,v 1.102 1998/04/04 13:11:51 assar Exp $ */
 
 #define LOGALL
 #define KERBEROS
@@ -66,6 +66,9 @@
 #include <stdarg.h>
 
 #include <errno.h>
+#ifdef HAVE_IO_H
+#include <io.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -372,5 +375,7 @@ __attribute__ ((format (printf, 1, 2)))
 
 char *clean_ttyname (char *tty);
 char *make_id (char *tty);
+#ifdef HAVE_UTMP_H
 void prepare_utmp (struct utmp *utmp, char *tty, char *username,
 		   char *hostname);
+#endif
