@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -40,7 +40,7 @@
 
 #ifndef HAVE_FORKPTY
 
-RCSID("$Id: forkpty.c,v 1.48 1996/11/23 17:11:03 joda Exp $");
+RCSID("$Id: forkpty.c,v 1.52 1997/05/25 07:37:01 assar Exp $");
 
 /* Only CRAY is known to have problems with forkpty(). */
 #if defined(CRAY)
@@ -116,12 +116,12 @@ static char *bsd_1 = "0123456789abcdefghijklmnopqrstuv";
 static char *bsd_2 = "pqrstuvwxyzabcdefghijklmnoABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 static int
-pty_scan_next(char *buf)
+pty_scan_next(char *buf, size_t sz)
 {
 #ifdef CRAY
     if(++pty_major >= sysconf(_SC_CRAY_NPTY))
 	return -1;
-    sprintf(buf, "/dev/pty/%03d", pty_major);
+    snprintf(buf, sz, "/dev/pty/%03d", pty_major);
 #else
     if(++pty_major == strlen(bsd_1)){
 	pty_major = 0;
@@ -129,25 +129,23 @@ pty_scan_next(char *buf)
 	    return -1;
     }
 #ifdef __hpux
-    sprintf(buf, "/dev/ptym/pty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
+    snprintf(buf, sz, "/dev/ptym/pty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
 #else
-    sprintf(buf, "/dev/pty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
-#endif
-#endif
+    snprintf(buf, sz, "/dev/pty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
+#endif /* __hpux */
+#endif /* CRAY */
     return 0;
 }
 
 static void
-pty_scan_tty(char *buf)
+pty_scan_tty(char *buf, size_t sz)
 {
 #ifdef CRAY
-    sprintf(buf, "/dev/ttyp%03d", pty_major);
+    snprintf(buf, sz, "/dev/ttyp%03d", pty_major);
+#elif defined(__hpux)
+    snprintf(buf, sz, "/dev/pty/tty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
 #else
-#ifdef __hpux
-    sprintf(buf, "/dev/pty/tty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
-#else
-    sprintf(buf, "/dev/tty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
-#endif
+    snprintf(buf, sz, "/dev/tty%c%c", bsd_2[pty_major], bsd_1[pty_minor]);
 #endif
 }
 
@@ -161,13 +159,13 @@ ptym_open_streams_flavor(char *pts_name, int *streams_pty)
     const char *const *q;
     
     for (q = clone; *q; q++) {
-      fdm = open(*q, O_RDWR);
-      if (fdm >= 0)
-	break;
+	fdm = open(*q, O_RDWR);
+	if (fdm >= 0)
+	    break;
     }
     if (fdm >= 0) {
-        char *ptr1;
-        if ( (ptr1 = ptsname(fdm)) != NULL) /* Get slave's name */
+	char *ptr1;
+	if ((ptr1 = ptsname(fdm)) != NULL) /* Get slave's name */
 	    strcpy(pts_name, ptr1); /* Return name of slave */  
 	else {
 	    close(fdm);
@@ -194,7 +192,7 @@ ptym_open_bsd_flavor(char *pts_name, int *streams_pty)
 
     pty_scan_start();
 
-    while (pty_scan_next(ptm) != -1) {
+    while (pty_scan_next(ptm, sizeof(ptm)) != -1) {
 	fdm = open(ptm, O_RDWR);
 	if (fdm < 0)
 	    continue;
@@ -211,7 +209,7 @@ ptym_open_bsd_flavor(char *pts_name, int *streams_pty)
 	    }
 	}
 #endif
-	pty_scan_tty(pts_name);
+	pty_scan_tty(pts_name, sizeof(ptm));
 #if CRAY
 	/* this is some magic from the telnet code */
 	{
@@ -258,10 +256,21 @@ ptym_open_bsd_flavor(char *pts_name, int *streams_pty)
  *
  */
 static int
-ptym_open(char *pts_name, int *streams_pty)
+ptym_open(char *pts_name, size_t pts_name_sz, int *streams_pty)
 {
     int	fdm;
 
+#ifdef HAVE__GETPTY
+    {
+	char *p = _getpty(&fdm, O_RDWR, 0600, 1);
+	if (p) {
+	    *streams_pty = 1;
+	    strcpy (pts_name, p);
+	    return fdm;
+	}
+    }
+#endif
+
 #ifdef STREAMSPTY
     fdm = ptym_open_streams_flavor(pts_name, streams_pty);
     if (fdm >= 0)
@@ -378,7 +387,7 @@ forkpty(int *ptrfdm,
     if (!forkpty_ok)
         fatal(0, "Protocol not yet supported, use telnet", 0);
 
-    if ( (fdm = ptym_open(pts_name, &streams_pty)) < 0)
+    if ( (fdm = ptym_open(pts_name, sizeof(pts_name), &streams_pty)) < 0)
 	return -1;
 
     if (slave_name != NULL)
@@ -427,10 +436,12 @@ forkpty(int *ptrfdm,
 	    if (tcsetattr(fds, TCSANOW, slave_termios) < 0)
 		return -1;
 	}
+#ifdef TIOCSWINSZ
 	if (slave_winsize != NULL) {
 	    if (ioctl(fds, TIOCSWINSZ, slave_winsize) < 0)
 		return -1;
 	}
+#endif
 	/* slave becomes stdin/stdout/stderr of child */
 	if (dup2(fds, STDIN_FILENO) != STDIN_FILENO)
 	    return -1;
