@@ -39,7 +39,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: login.c,v 1.109 1997/12/09 10:36:26 joda Exp $");
+RCSID("$Id: login.c,v 1.112 1998/05/18 19:28:31 joda Exp $");
 
 #ifdef OTP
 #include <otp.h>
@@ -698,7 +698,9 @@ main(int argc, char **argv)
 	signal(SIGALRM, SIG_DFL);
 	signal(SIGQUIT, SIG_DFL);
 	signal(SIGINT, SIG_DFL);
+#ifdef SIGTSTP
 	signal(SIGTSTP, SIG_IGN);
+#endif
 
 	tbuf[0] = '-';
 	strcpy(tbuf + 1, (p = strrchr(pwd->pw_shell, '/')) ?
@@ -757,8 +759,16 @@ main(int argc, char **argv)
 
 	if (k_hasafs()) {
 	    char cell[64];
+#ifdef _AIX
+	    /* XXX this is a fix for a bug in AFS for AIX 4.3, w/o
+               this hack the kernel crashes on the following
+               pioctl... */
+	    char *pw_dir = strdup(pwd->pw_dir);
+#else
+	    char *pw_dir = pwd->pw_dir;
+#endif
 	    k_setpag();
-	    if(k_afs_cell_of_file(pwd->pw_dir, cell, sizeof(cell)) == 0)
+	    if(k_afs_cell_of_file(pw_dir, cell, sizeof(cell)) == 0)
 		krb_afslog(cell, 0);
 	    krb_afslog(0, 0);
 	}
@@ -813,16 +823,45 @@ getloginname(int prompt)
 	}
 }
 
+static int
+find_in_etc_securetty (char *ttyn)
+{
+    FILE *f;
+    char buf[128];
+    int ret = 0;
+
+    f = fopen (_PATH_ETC_SECURETTY, "r");
+    if (f == NULL)
+	return 0;
+    while (fgets(buf, sizeof(buf), f) != NULL) {
+	if(buf[strlen(buf) - 1] == '\n')
+	    buf[strlen(buf) - 1] = '\0';
+	if (strcmp (buf, ttyn) == 0) {
+	    ret = 1;
+	    break;
+	}
+    }
+    fclose(f);
+    return ret;
+}
+
 static int
 rootterm(char *ttyn)
 {
-#ifndef HAVE_TTYENT_H
-        return (default_console == 0 || strcmp(default_console, ttyname(0)) == 0);
-#else
+#ifdef HAVE_TTYENT_H
+    {
 	struct ttyent *t;
 
-	return ((t = getttynam(ttyn)) && t->ty_status & TTY_SECURE);
+	t = getttynam (ttyn);
+	if (t && t->ty_status & TTY_SECURE)
+	    return 1;
+    }
 #endif
+    if (find_in_etc_securetty(ttyn))
+	return 1;
+    if (default_console == 0 || strcmp(default_console, ttyn) == 0)
+	return 1;
+    return 0;
 }
 
 static RETSIGTYPE
