@@ -42,7 +42,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: rshd.c,v 1.52 1997/12/09 10:36:30 joda Exp $");
+RCSID("$Id: rshd.c,v 1.53 1998/02/10 21:12:10 bg Exp $");
 
 extern char *__rcmd_errstr; /* syslog hook from libc/net/rcmd.c. */
 extern int __check_rhosts_file;
@@ -279,7 +279,7 @@ doit(struct sockaddr_in *fromp)
     }
 
     if (vacuous) {
-	error("rshd: remote host requires Kerberos authentication\n");
+	error("rshd: Remote host requires Kerberos authentication.\n");
 	exit(1);
     }
 
@@ -298,7 +298,7 @@ doit(struct sockaddr_in *fromp)
 	    if (getsockname(0, (struct sockaddr *)&local_addr,
 			    &rc) < 0) {
 		syslog(LOG_ERR, "getsockname: %m");
-		error("rlogind: getsockname: %m");
+		error("rshd: getsockname: %m");
 		exit(1);
 	    }
 	    authopts = KOPT_DO_MUTUAL;
@@ -620,8 +620,8 @@ error(const char *fmt, ...)
 	len = 1;
     } else
 	len = 0;
-    len = vsnprintf (bp, sizeof(buf) - len, fmt, ap);
-    write (STDERR_FILENO, buf, len);
+    len += vsnprintf(bp, sizeof(buf) - len, fmt, ap);
+    write(STDERR_FILENO, buf, len);
     va_end(ap);
 }
 
