@@ -33,7 +33,7 @@
 
 #include "bsd_locl.h"
 
-RCSID ("$Id: su.c,v 1.58 1997/05/02 14:27:49 assar Exp $");
+RCSID ("$Id: su.c,v 1.63 1997/12/09 10:36:30 joda Exp $");
 
 #ifdef SYSV_SHADOW
 #include "sysv_shadow.h"
@@ -123,6 +123,8 @@ main (int argc, char **argv)
     if (pwd == NULL)
 	errx (1, "who are you?");
     username = strdup (pwd->pw_name);
+    if (username == NULL)
+	errx (1, "strdup: out of memory");
     if (asme)
 	if (pwd->pw_shell && *pwd->pw_shell)
 	    shell = strcpy (shellbuf, pwd->pw_shell);
@@ -229,6 +231,8 @@ main (int argc, char **argv)
 	    char *t = getenv ("TERM");
 
 	    environ = malloc (10 * sizeof (char *));
+	    if (environ == NULL)
+		err (1, "malloc");
 	    environ[0] = NULL;
 	    setenv ("PATH", _PATH_DEFPATH, 1);
 	    if (t)
@@ -272,7 +276,7 @@ main (int argc, char **argv)
 
 	if (k_setpag () != 0)
 	    warn ("setpag");
-	code = k_afsklog (0, 0);
+	code = krb_afslog (0, 0);
 	if (code != KSUCCESS && code != KDC_PR_UNKNOWN)
 	    warnx ("afsklog: %s", krb_get_err_text (code));
     }
@@ -333,6 +337,15 @@ kerberos (char *username, char *user, int uid)
 
     setenv ("KRBTKFILE", krbtkfile, 1);
     krb_set_tkt_string (krbtkfile);
+    /*
+     * Set real as well as effective ID to 0 for the moment,
+     * to make the kerberos library do the right thing.
+     */
+    if (setuid(0) < 0) {
+        warn("setuid");
+	return (1);
+    }
+
     /*
      * Little trick here -- if we are su'ing to root, we need to get a ticket
      * for "xxx.root", where xxx represents the name of the person su'ing.
@@ -354,8 +367,10 @@ kerberos (char *username, char *user, int uid)
 					 lrealm));
 	if (des_read_pw_string (passw, sizeof (passw), prompt, 0)) {
 	    memset (passw, 0, sizeof (passw));
-	    return 1;
+	    return (1);
 	}
+	if (strlen(passw) == 0)
+	    return (1);		/* Empty passwords is not allowed */
 	kerno = krb_get_pw_in_tkt ((uid == 0 ? username : user),
 				   (uid == 0 ? root_inst : ""), lrealm,
 				   KRB_TICKET_GRANTING_TICKET,
@@ -386,7 +401,7 @@ kerberos (char *username, char *user, int uid)
     }
     setpriority (PRIO_PROCESS, 0, -2);
 
-    if (k_gethostname (hostname, sizeof (hostname)) == -1) {
+    if (gethostname (hostname, sizeof (hostname)) == -1) {
 	warn ("gethostname");
 	dest_tkt ();
 	return (1);
