@@ -2,7 +2,7 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: utmpx_login.c,v 1.18 1997/05/20 20:35:08 assar Exp $");
+RCSID("$Id: utmpx_login.c,v 1.20 1997/06/01 03:13:15 assar Exp $");
 
 /* utmpx_login - update utmp and wtmp after login */
 
@@ -14,8 +14,12 @@ static void
 utmpx_update(struct utmpx *ut, char *line, char *user, char *host)
 {
     struct timeval tmp;
+    char *clean_tty = clean_ttyname(line);
 
-    strncpy(ut->ut_line, clean_ttyname(line), sizeof(ut->ut_line));
+    strncpy(ut->ut_line, clean_tty, sizeof(ut->ut_line));
+#ifdef HAVE_UT_ID
+    strncpy(ut->ut_id, make_id(clean_tty), sizeof(ut->ut_id));
+#endif
     strncpy(ut->ut_user, user, sizeof(ut->ut_user));
     strncpy(ut->ut_host, host, sizeof(ut->ut_host));
 #ifdef HAVE_UT_SYSLEN
