@@ -1,5 +1,3 @@
-/*      $NetBSD: domacro.c,v 1.5 1995/09/08 01:06:14 tls Exp $      */
-
 /*
  * Copyright (c) 1985, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -34,7 +32,7 @@
  */
 
 #include "ftp_locl.h"
-RCSID("$Id: domacro.c,v 1.3 1996/08/26 22:46:22 assar Exp $");
+RCSID("$Id: domacro.c,v 1.5 1996/11/17 20:23:10 assar Exp $");
 
 void
 domacro(int argc, char **argv)
@@ -58,7 +56,7 @@ domacro(int argc, char **argv)
 		code = -1;
 		return;
 	}
-	(void) strcpy(line2, line);
+	strcpy(line2, line);
 TOP:
 	cp1 = macros[i].mac_start;
 	while (cp1 != macros[i].mac_end) {
@@ -79,7 +77,7 @@ TOP:
 				    }
 				    cp1--;
 				    if (argc - 2 >= j) {
-					(void) strcpy(cp2, argv[j+1]);
+					strcpy(cp2, argv[j+1]);
 					cp2 += strlen(argv[j+1]);
 				    }
 				    break;
@@ -88,7 +86,7 @@ TOP:
 					loopflg = 1;
 					cp1++;
 					if (count < argc) {
-					   (void) strcpy(cp2, argv[count]);
+					   strcpy(cp2, argv[count]);
 					   cp2 += strlen(argv[count]);
 					}
 					break;
@@ -123,9 +121,9 @@ TOP:
 			}
 			(*c->c_handler)(margc, margv);
 			if (bell && c->c_bell) {
-				(void) putchar('\007');
+				putchar('\007');
 			}
-			(void) strcpy(line, line2);
+			strcpy(line, line2);
 			makeargv();
 			argc = margc;
 			argv = margv;
