@@ -32,7 +32,7 @@
  */
 
 #include "ftp_locl.h"
-RCSID("$Id: ftp.c,v 1.44 1997/05/18 20:00:31 assar Exp $");
+RCSID("$Id: ftp.c,v 1.47 1997/12/09 13:24:58 joda Exp $");
 
 struct	sockaddr_in hisctladdr;
 struct	sockaddr_in data_addr;
@@ -555,7 +555,7 @@ copy_stream(FILE *from, FILE *to)
     int hashbytes = HASHBYTES;
     struct stat st;
     
-#ifdef HAVE_MMAP
+#if defined(HAVE_MMAP) && !defined(NO_MMAP)
     void *chunk;
 
 #ifndef MAP_FAILED
@@ -616,7 +616,7 @@ sendrequest(char *cmd, char *local, char *remote, int printnames)
     char *lmode;
 
     if (verbose && printnames) {
-	if (local && *local != '-')
+	if (local && strcmp(local, "-") != 0)
 	    printf("local: %s ", local);
 	if (remote)
 	    printf("remote: %s\n", remote);
@@ -832,7 +832,8 @@ abortrecv(int sig)
 }
 
 void
-recvrequest(char *cmd, char *local, char *remote, char *lmode, int printnames)
+recvrequest(char *cmd, char *local, char *remote, 
+	    char *lmode, int printnames, int local_given)
 {
     FILE *fout, *din = 0;
     int (*closefunc) (FILE *);
@@ -846,7 +847,7 @@ recvrequest(char *cmd, char *local, char *remote, char *lmode, int printnames)
 
     is_retr = strcmp(cmd, "RETR") == 0;
     if (is_retr && verbose && printnames) {
-	if (local && *local != '-')
+	if (local && strcmp(local, "-") != 0)
 	    printf("local: %s ", local);
 	if (remote)
 	    printf("remote: %s\n", remote);
@@ -873,7 +874,7 @@ recvrequest(char *cmd, char *local, char *remote, char *lmode, int printnames)
 	return;
     }
     oldintr = signal(SIGINT, abortrecv);
-    if (strcmp(local, "-") && *local != '|') {
+    if (!local_given || (strcmp(local, "-") && *local != '|')) {
 	if (access(local, 2) < 0) {
 	    char *dir = strrchr(local, '/');
 
@@ -945,9 +946,9 @@ recvrequest(char *cmd, char *local, char *remote, char *lmode, int printnames)
     if (din == NULL)
 	goto abort;
     set_buffer_size(fileno(din), 1);
-    if (strcmp(local, "-") == 0)
+    if (local_given && strcmp(local, "-") == 0)
 	fout = stdout;
-    else if (*local == '|') {
+    else if (local_given && *local == '|') {
 	oldintp = signal(SIGPIPE, SIG_IGN);
 	fout = popen(local + 1, "w");
 	if (fout == NULL) {
