@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -36,14 +36,11 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kauth.h,v 1.12 1996/10/27 16:49:59 joda Exp $ */
+/* $Id: kauth.h,v 1.18 1997/05/20 18:40:31 bg Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
-#ifdef SOCKS
-#include <socks.h>
-#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,8 +61,13 @@
 #include <grp.h>
 #endif
 
-#ifdef HAVE_SYS_TIME_H
+#ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
+#include <time.h>
+#elif defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#else
+#include <time.h>
 #endif
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
@@ -88,6 +90,11 @@
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
+#ifdef SOCKS
+#include <socks.h>
+#endif
+
+#include <err.h>
 
 #include <krb.h>
 #include <kafs.h>
@@ -96,19 +103,16 @@
 
 #define KAUTH_PORT 2120
 
-int rkinit (char *name, char *inst, char *realm, int lifetime,
-	    char *locuser, char *tktfile, des_cblock *key, char *hostname);
+#define KAUTH_VERSION "RKINIT.0"
 
-int write_encrypted (int fd, void *buf, size_t len,
-		     des_key_schedule schedule, des_cblock *session,
-		     struct sockaddr_in *me, struct sockaddr_in *her);
+int rkinit (krb_principal*, int, char*, char*, des_cblock*, char*);
 
-int read_encrypted (int fd, void *buf, size_t len, void **ret,
-		    des_key_schedule schedule, des_cblock *session,
-		    struct sockaddr_in *her, struct sockaddr_in *me);
+int write_encrypted (int, void*, size_t, des_key_schedule,
+		     des_cblock*, struct sockaddr_in*, struct sockaddr_in*);
 
-unsigned pack_args (char *buf, char *name, char *inst, char *realm,
-		    int lifetime, char *locuser, char *tktfile);
+int read_encrypted (int, void*, size_t, void **, des_key_schedule,
+		    des_cblock*, struct sockaddr_in*, struct sockaddr_in*);
 
-int unpack_args (char *buf, char *name, char *inst, char *realm,
-		 int *lifetime, char *locuser, char *tktfile);
+unsigned pack_args (char *, krb_principal*, int, char*, char*);
+
+int unpack_args (char*, krb_principal*, int*, char*, char*);
