@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,11 +38,7 @@
 
 #include "kip.h"
 
-RCSID("$Id: kipd.c,v 1.6 1996/10/27 16:50:05 joda Exp $");
-
-#include <syslog.h>
-
-char *progname;
+RCSID("$Id: kipd.c,v 1.13 1997/05/18 20:38:01 assar Exp $");
 
 static int
 fatal (int fd, char *s)
@@ -65,7 +61,7 @@ recv_conn (int sock, des_cblock *key, des_key_schedule schedule,
      char instance[INST_SZ + 1];
      struct sockaddr_in thisaddr, thataddr;
      int addrlen;
-     char version[KRB_SENDAUTH_VLEN];
+     char version[KRB_SENDAUTH_VLEN + 1];
      u_char ok = 0;
      struct passwd *passwd;
 
@@ -80,12 +76,12 @@ recv_conn (int sock, des_cblock *key, des_key_schedule schedule,
 	  return 1;
      }
 
-     k_getsockinst (sock, instance);
+     k_getsockinst (sock, instance, sizeof(instance));
      status = krb_recvauth (KOPT_DO_MUTUAL, sock, &ticket, "rcmd", instance,
 			    &thataddr, &thisaddr, &auth, "", schedule,
 			    version);
      if (status != KSUCCESS ||
-	 strncmp(version, "KIPSRV.0", KRB_SENDAUTH_VLEN) != 0) {
+	 strncmp(version, KIP_VERSION, KRB_SENDAUTH_VLEN) != 0) {
 	  return 1;
      }
      passwd = k_getpwnam ("root");
@@ -124,9 +120,9 @@ doit(int sock)
 int
 main (int argc, char **argv)
 {
-     progname = argv[0];
+    set_progname (argv[0]);
 
-     openlog(progname, LOG_PID|LOG_CONS, LOG_DAEMON);
-     signal (SIGCHLD, childhandler);
-     return doit(0);
+    openlog(__progname, LOG_PID|LOG_CONS, LOG_DAEMON);
+    signal (SIGCHLD, childhandler);
+    return doit(0);
 }
