@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: kx.c,v 1.38 1997/05/18 18:53:06 assar Exp $");
+RCSID("$Id: kx.c,v 1.46 1998/05/28 15:20:33 joda Exp $");
 
 static int nchild;
 static int donep;
@@ -114,12 +114,7 @@ connect_host (char *host, char *user, des_cblock *key,
      hostent = gethostbyname (host);
      if (hostent == NULL) {
 	 warnx ("gethostbyname '%s' failed: %s", host,
-#ifdef HAVE_H_ERRNO
-		hstrerror(h_errno)
-#else
-		"unknown error"
-#endif
-	     );
+		hstrerror(h_errno));
 	 return -1;
      }
 
@@ -353,13 +348,16 @@ doit_active (char *host, char *user,
     des_key_schedule schedule;
     des_cblock key;
     int otherside;
-    int rendez_vous1 = 0, rendez_vous2 = 0;
+    int nsockets;
+    struct x_socket *sockets;
     struct sockaddr_in me, him;
     u_char msg[1024], *p;
     int len = strlen(user);
     void *ret;
     u_int32_t tmp;
+    int tmp2;
     char *s;
+    int i;
 
     otherside = connect_host (host, user, &key, schedule, port,
 			      &me, &him);
@@ -415,39 +413,33 @@ doit_active (char *host, char *user,
     } else
 	p++;
 
-    tmp = get_xsockets (&rendez_vous1,
-			tcpp ? &rendez_vous2 : NULL);
-    if (tmp < 0)
+    tmp2 = get_xsockets (&nsockets, &sockets, tcpp);
+    if (tmp2 < 0)
 	return 1;
-    display_num = tmp;
+    display_num = tmp2;
     if (tcpp)
 	snprintf (display, display_size, "localhost:%u", display_num);
     else
 	snprintf (display, display_size, ":%u", display_num);
-    strncpy(xauthfile, tempnam("/tmp", NULL), xauthfile_size);
-    if (create_and_write_cookie (xauthfile, cookie, cookie_len))
+    if (create_and_write_cookie (xauthfile, xauthfile_size, cookie, cookie_len))
 	return 1;
     status_output (debugpp);
     for (;;) {
 	fd_set fdset;
 	pid_t child;
-	int fd, thisfd;
+	int fd, thisfd = -1;
 	int zero = 0;
 
 	FD_ZERO(&fdset);
-	if (rendez_vous1)
-	    FD_SET(rendez_vous1, &fdset);
-	if (rendez_vous2)
-	    FD_SET(rendez_vous2, &fdset);
+	for (i = 0; i < nsockets; ++i)
+	    FD_SET(sockets[i].fd, &fdset);
 	if (select(FD_SETSIZE, &fdset, NULL, NULL, NULL) <= 0)
 	    continue;
-	if (rendez_vous1 && FD_ISSET(rendez_vous1, &fdset))
-	    thisfd = rendez_vous1;
-	else if (rendez_vous2 && FD_ISSET(rendez_vous2, &fdset))
-	    thisfd = rendez_vous2;
-	else
-	    continue;
-
+	for (i = 0; i < nsockets; ++i)
+	    if (FD_ISSET(sockets[i].fd, &fdset)) {
+		thisfd = sockets[i].fd;
+		break;
+	    }
 	fd = accept (thisfd, NULL, &zero);
 	if (fd < 0)
 	    if (errno == EINTR)
@@ -485,10 +477,8 @@ doit_active (char *host, char *user,
 	    int s;
 	    struct sockaddr_in addr;
 
-	    if (rendez_vous1)
-		close (rendez_vous1);
-	    if (rendez_vous2)
-		close (rendez_vous2);
+	    for (i = 0; i < nsockets; ++i)
+		close (sockets[i].fd);
 
 	    addr = him;
 	    close (otherside);
@@ -524,35 +514,56 @@ doit_active (char *host, char *user,
     }
 }
 
+/*
+ *
+ */
+
+static int
+check_for_passive (const char *disp)
+{
+    char local_hostname[MaxHostNameLen];
+
+    gethostname (local_hostname, sizeof(local_hostname));
+
+    return disp != NULL &&
+	(*disp == ':'
+	 || strncmp(disp, "unix", 4) == 0
+	 || strncmp(disp, "localhost", 9) == 0
+	 || strncmp(disp, local_hostname, strlen(local_hostname) == 0));
+}
+
 static void
 usage(void)
 {
-    fprintf (stderr, "Usage: %s [-p port] [-d] [-t] [-l remoteuser] host\n",
-	     __progname);
+    fprintf(stderr, "Usage: %s [-p port] [-d] [-D] [-t] [-l remoteuser] host\n",
+	    __progname);
     exit (1);
 }
 
 /*
  * kx - forward x connection over a kerberos-encrypted channel.
  *
- * passive mode if $DISPLAY begins with : */
+ */
 
 int
 main(int argc, char **argv)
 {
-     int passivep;
+     int force_passive = 0;
      int keepalivep = 1;
-     char *disp, *user = NULL;
+     char *user = NULL;
      int debugp = 0, tcpp = 0;
      int c;
      int port = 0;
 
      set_progname (argv[0]);
-     while((c = getopt(argc, argv, "ktdl:p:")) != EOF) {
+     while((c = getopt(argc, argv, "ktdDl:p:P")) != EOF) {
 	 switch(c) {
 	 case 'd' :
 	     debugp = 1;
 	     break;
+	 case 'D':
+	     krb_enable_debug();
+	     break;
 	 case 'k':
 	     keepalivep = 0;
 	     break;
@@ -565,6 +576,9 @@ main(int argc, char **argv)
 	 case 'p' :
 	     port = htons(atoi (optarg));
 	     break;
+	 case 'P' :
+	     force_passive = 1;
+	     break;
 	 case '?':
 	 default:
 	     usage();
@@ -581,16 +595,15 @@ main(int argc, char **argv)
 	  if (p == NULL)
 	      errx(1, "Who are you?");
 	  user = strdup (p->pw_name);
+	  if (user == NULL)
+	      errx (1, "strdup: out of memory");
      }
      if (port == 0)
 	 port = k_getportbyname ("kx", "tcp", htons(KX_PORT));
-     disp = getenv("DISPLAY");
-     passivep = disp != NULL && 
-       (*disp == ':' || strncmp(disp, "unix", 4) == 0);
      signal (SIGCHLD, childhandler);
      signal (SIGUSR1, usr1handler);
      signal (SIGUSR2, usr2handler);
-     if (passivep)
+     if (check_for_passive(getenv("DISPLAY")))
 	 return doit_passive (argv[0], user, debugp, keepalivep, port);
      else
 	 return doit_active  (argv[0], user, debugp, keepalivep, tcpp, port);
