@@ -55,10 +55,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
-#ifdef SOCKS
-#include <socks.h>
+#include <protos.h>
 #endif
 
 #include <stdio.h>
@@ -75,8 +72,13 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
+#ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
+#include <time.h>
+#elif defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#else
+#include <time.h>
 #endif
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
@@ -105,6 +107,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #if 0
 #include "syswait.h"
 #endif
+#ifdef SOCKS
+#include <socks.h>
+#endif
+#include <roken.h>
+
 #include "pop.h"
 
 #ifdef MSDOS
