@@ -23,7 +23,7 @@
 
 #include "sample.h"
 
-RCSID("$Id: sample_client.c,v 1.11 1997/03/30 17:38:23 assar Exp $");
+RCSID("$Id: sample_client.c,v 1.13 1998/05/01 05:20:43 assar Exp $");
 
 static void
 usage (void)
@@ -98,15 +98,12 @@ main(int argc, char **argv)
     hp = gethostbyname(argv[0]);
     if (hp == NULL)
 	errx (1, "gethostbyname(%s): %s", argv[0],
-#ifdef HAVE_H_ERRNO
-	      hstrerror(h_errno)
-#else
-	      "unknown error"
-#endif
-	       );
+	      hstrerror(h_errno));
 
     /* copy the hostname into non-volatile storage */
     remote_host = strdup(hp->h_name);
+    if (remote_host == NULL)
+	errx (1, "strdup: out of memory");
 
     /* set up the address of the foreign socket for connect() */
     sin.sin_family = hp->h_addrtype;
