@@ -18,14 +18,13 @@
 
 #include "sample.h"
 
-RCSID("$Id: sample_server.c,v 1.6 1996/09/27 15:54:21 assar Exp $");
-
-static char *prog;
+RCSID("$Id: sample_server.c,v 1.10 1997/05/02 14:28:20 assar Exp $");
 
 static void
 usage (void)
 {
-    fprintf (stderr, "Usage: %s [-i] [-s service] [-t srvtab]\n", prog);
+    fprintf (stderr, "Usage: %s [-i] [-s service] [-t srvtab]\n",
+	     __progname);
     exit (1);
 }
 
@@ -50,9 +49,9 @@ main(int argc, char **argv)
 
     /* open a log connection */
 
-    prog = argv[0];
+    set_progname (argv[0]);
 
-    openlog(prog, LOG_ODELAY, LOG_DAEMON);
+    openlog(__progname, LOG_ODELAY, LOG_DAEMON);
 
     strcpy (service, SAMPLE_SERVICE);
     *srvtab = '\0';
@@ -74,7 +73,7 @@ main(int argc, char **argv)
 	}
 
     if (no_inetd)
-	mini_inetd (SAMPLE_PORT);
+	mini_inetd (htons(SAMPLE_PORT));
 
     /*
      * To verify authenticity, we need to know the address of the
@@ -97,7 +96,7 @@ main(int argc, char **argv)
     /* read the authenticator and decode it.  Using `k_getsockinst' we
      * always get the right instance on a multi-homed host.
      */
-    k_getsockinst (STDIN_FILENO, instance);
+    k_getsockinst (STDIN_FILENO, instance, sizeof(instance));
 
     /* we want mutual authentication */
     authopts = KOPT_DO_MUTUAL;
@@ -106,9 +105,10 @@ main(int argc, char **argv)
 			  &auth_data, srvtab,
 			  sched, version);
     if (status != KSUCCESS) {
-	syslog(LOG_ERR, "Kerberos error: %s\n", krb_get_err_text(status));
-	sprintf(retbuf, "Kerberos error: %s\n",
-		krb_get_err_text(status));
+	snprintf(retbuf, sizeof(retbuf),
+		 "Kerberos error: %s\n",
+		 krb_get_err_text(status));
+	syslog(LOG_ERR, retbuf);
     } else {
 	/* Check the version string (KRB_SENDAUTH_VLEN chars) */
 	if (strncmp(version, SAMPLE_VERSION, KRB_SENDAUTH_VLEN)) {
@@ -125,7 +125,7 @@ main(int argc, char **argv)
 	    strcpy(lname,
 		   "*No local name returned by krb_kntoln*");
 	/* compose the reply */
-	sprintf(retbuf,
+	snprintf(retbuf, sizeof(retbuf),
 		"You are %s.%s@%s (local name %s),\n at address %s, version %s, cksum %ld\n",
 		auth_data.pname,
 		auth_data.pinst,
