@@ -10,7 +10,7 @@
  */
 
 #include "sample.h"
-RCSID("$Id: simple_client.c,v 1.11 1997/03/30 17:38:24 assar Exp $");
+RCSID("$Id: simple_client.c,v 1.12 1997/12/05 01:26:56 assar Exp $");
 
 #define MSG "hi, Jennifer!"		/* message text */
 
@@ -54,7 +54,7 @@ talkto(char *hostname, char *service, int port)
   else
     s_sock.sin_port = k_getportbyname (service, "tcp", htons(SAMPLE_PORT));
 
-  if (k_gethostname(myhostname, sizeof(myhostname)) < 0) {
+  if (gethostname(myhostname, sizeof(myhostname)) < 0) {
     warn("gethostname");
     return 1;
   }
