@@ -11,14 +11,13 @@
 
 #include "sample.h"
 
-RCSID("$Id: simple_server.c,v 1.7 1996/09/27 15:54:25 assar Exp $");
-
-static char *prog;
+RCSID("$Id: simple_server.c,v 1.8 1997/03/30 17:38:25 assar Exp $");
 
 static void
 usage (void)
 {
-    fprintf (stderr, "Usage: %s [-p port] [-s service] [-t srvtab]\n", prog);
+    fprintf (stderr, "Usage: %s [-p port] [-s service] [-t srvtab]\n",
+	     __progname);
     exit (1);
 }
 
@@ -40,7 +39,7 @@ main(int argc, char **argv)
     MSG_DAT msg_data;
     des_key_schedule sched;
 
-    prog = argv[0];
+    set_progname (argv[0]);
     strcpy (service, SAMPLE_SERVICE);
     strcpy (instance, "*");
     *srvtab = '\0';
@@ -79,32 +78,25 @@ main(int argc, char **argv)
     addr.sin_port = port;
 
     sock = socket (AF_INET, SOCK_DGRAM, 0);
-    if (sock < 0) {
-	perror ("socket");
-	return 1;
-    }
+    if (sock < 0)
+	err (1, "socket");
 
-    if (bind (sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-	perror ("bind");
-	return 1;
-    }
+    if (bind (sock, (struct sockaddr *)&addr, sizeof(addr)) < 0)
+	err (1, "bind");
 
     /* GET KRB_MK_REQ MESSAGE */
 
     i = read(sock, ktxt->dat, MAX_KTXT_LEN);
-    if (i < 0) {
-	perror("receiving datagram");
-	return 1;
-    }
+    if (i < 0)
+	err (1, "read");
+
     printf("Received %d bytes\n", i);
     ktxt->length = i;
 
     /* Check authentication info */
     i = krb_rd_req(ktxt, service, instance, 0, &ad, "");
-    if (i != KSUCCESS) {
-	fprintf(stderr, "%s\n", krb_get_err_text(i));
-	exit(1);
-    }
+    if (i != KSUCCESS)
+	errx (1, "krb_rd_req: %s", krb_get_err_text(i));
     printf("Got authentication info from %s%s%s@%s\n", ad.pname,
 	   *ad.pinst ? "." : "", ad.pinst, ad.prealm);
 	
@@ -114,19 +106,15 @@ main(int argc, char **argv)
     len = sizeof(otheraddr);
     i = recvfrom(sock, ktxt->dat, MAX_KTXT_LEN, 0,
 		 (struct sockaddr *)&otheraddr, &len);
-    if (i < 0) {
-	perror("receiving datagram");
-	return 1;
-    }
+    if (i < 0)
+	err (1, "recvfrom");
     printf("Received %d bytes\n", i);
 
     /* Verify the checksummed message */
     i = krb_rd_safe(ktxt->dat, i, &ad.session, &otheraddr,
 		    &addr, &msg_data);
-    if (i != KSUCCESS) {
-	fprintf(stderr, "%s\n", krb_get_err_text(i));
-	exit(1);
-    }
+    if (i != KSUCCESS)
+	errx (1, "krb_rd_safe: %s", krb_get_err_text(i));
     printf("Safe message is: %s\n", msg_data.app_data);
 	
     /* NOW GET ENCRYPTED MESSAGE */
@@ -142,17 +130,13 @@ main(int argc, char **argv)
     len = sizeof(otheraddr);
     i = recvfrom(sock, ktxt->dat, MAX_KTXT_LEN, 0,
 		 (struct sockaddr *)&otheraddr, &len);
-    if (i < 0) {
-	perror("receiving datagram");
-	return 1;
-    }
+    if (i < 0)
+	err (1, "recvfrom");
     printf("Received %d bytes\n", i);
     i = krb_rd_priv(ktxt->dat, i, sched, &ad.session, &otheraddr,
 		    &addr, &msg_data);
-    if (i != KSUCCESS) {
-	fprintf(stderr, "%s\n", krb_get_err_text(i));
-	return 1;
-    }
+    if (i != KSUCCESS)
+	errx (1, "krb_rd_priv: %s", krb_get_err_text(i));
     printf("Decrypted message is: %s\n", msg_data.app_data);
     return(0);
 }
