@@ -52,26 +52,26 @@
  */
 
 #include <config.h>
+
+RCSID("$Id: auth.c,v 1.18 1997/11/02 03:56:31 assar Exp $");
+
+#if	defined(AUTHENTICATION)
+#include <stdio.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#include <signal.h>
+#define	AUTH_NAMES
+#ifdef HAVE_ARPA_TELNET_H
+#include <arpa/telnet.h>
+#endif
+#include <stdlib.h>
+#include <string.h>
 #ifdef SOCKS
 #include <socks.h>
 #endif
 
-RCSID("$Id: auth.c,v 1.11 1996/11/19 22:48:32 assar Exp $");
-
-#if	defined(AUTHENTICATION)
-#include <stdio.h>
-#include <sys/types.h>
-#include <signal.h>
-#define	AUTH_NAMES
-#include <arpa/telnet.h>
-#ifdef	__STDC__
-#include <stdlib.h>
-#endif
-#ifdef	NO_STRING_H
-#include <strings.h>
-#else
-#include <string.h>
-#endif
+#include <roken.h>
 
 #include "encrypt.h"
 #include "auth.h"
@@ -148,9 +148,16 @@ Authenticator authenticators[] = {
       spx_printsub },
 #endif
 #ifdef	KRB5
+    { AUTHTYPE_KERBEROS_V5, AUTH_WHO_CLIENT|AUTH_HOW_MUTUAL,
+      kerberos5_init,
+      kerberos5_send_mutual,
+      kerberos5_is,
+      kerberos5_reply,
+      kerberos5_status,
+      kerberos5_printsub },
     { AUTHTYPE_KERBEROS_V5, AUTH_WHO_CLIENT|AUTH_HOW_ONE_WAY,
       kerberos5_init,
-      kerberos5_send,
+      kerberos5_send_oneway,
       kerberos5_is,
       kerberos5_reply,
       kerberos5_status,
@@ -493,7 +500,7 @@ auth_is(unsigned char *data, int cnt)
 	return;
     }
 
-    if (ap = findauthenticator(data[0], data[1])) {
+    if ((ap = findauthenticator(data[0], data[1]))) {
 	if (ap->is)
 	    (*ap->is)(ap, data+2, cnt-2);
     } else if (auth_debug_mode)
@@ -509,7 +516,7 @@ auth_reply(unsigned char *data, int cnt)
     if (cnt < 2)
 	return;
 
-    if (ap = findauthenticator(data[0], data[1])) {
+    if ((ap = findauthenticator(data[0], data[1]))) {
 	if (ap->reply)
 	    (*ap->reply)(ap, data+2, cnt-2);
     } else if (auth_debug_mode)
@@ -520,7 +527,7 @@ auth_reply(unsigned char *data, int cnt)
 void
 auth_name(unsigned char *data, int cnt)
 {
-    unsigned char savename[256];
+    char savename[256];
 
     if (cnt < 1) {
 	if (auth_debug_mode)
@@ -637,7 +644,7 @@ auth_gen_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
     buf[buflen-2] = '*';
     buflen -= 2;
     for (; cnt > 0; cnt--, data++) {
-	sprintf((char *)tbuf, " %d", *data);
+	snprintf(tbuf, sizeof(tbuf), " %d", *data);
 	for (cp = tbuf; *cp && buflen > 0; --buflen)
 	    *buf++ = *cp++;
 	if (buflen <= 0)
