@@ -53,11 +53,8 @@
 
 
 #include <config.h>
-#ifdef SOCKS
-#include <socks.h>
-#endif
 
-RCSID("$Id: encrypt.c,v 1.13 1996/11/17 20:04:24 assar Exp $");
+RCSID("$Id: encrypt.c,v 1.19 1997/11/02 03:58:03 assar Exp $");
 
 #if	defined(ENCRYPTION)
 
@@ -67,21 +64,21 @@ RCSID("$Id: encrypt.c,v 1.13 1996/11/17 20:04:24 assar Exp $");
 #include "encrypt.h"
 #include "misc.h"
 
-#ifdef	__STDC__
+#include <stdio.h>
 #include <stdlib.h>
-#endif
-#ifdef	NO_STRING_H
-#include <strings.h>
-#else
 #include <string.h>
+#ifdef SOCKS
+#include <socks.h>
 #endif
 
+#include <roken.h>
+
 /*
  * These functions pointers point to the current routines
  * for encrypting and decrypting data.
  */
-void	(*encrypt_output) P((unsigned char *, int));
-int	(*decrypt_input) P((int));
+void	(*encrypt_output) (unsigned char *, int);
+int	(*decrypt_input) (int);
 char	*nclearto;
 
 int encrypt_debug_mode = 0;
@@ -658,7 +655,7 @@ encrypt_start(unsigned char *data, int cnt)
 	return;
     }
 
-    if (ep = finddecryption(decrypt_mode)) {
+    if ((ep = finddecryption(decrypt_mode))) {
 	decrypt_input = ep->input;
 	if (encrypt_verbose)
 	    printf("[ Input is now decrypted with type %s ]\r\n",
@@ -732,7 +729,8 @@ encrypt_request_start(unsigned char *data, int cnt)
 
 static unsigned char str_keyid[(MAXKEYLEN*2)+5] = { IAC, SB, TELOPT_ENCRYPT };
 
-void encrypt_keyid(struct key_info *kp, unsigned char *keyid, int len)
+static void
+encrypt_keyid(struct key_info *kp, unsigned char *keyid, int len)
 {
     Encryptions *ep;
     int dir = kp->dir;
@@ -942,6 +940,18 @@ void encrypt_wait(void)
 	    return;
 }
 
+int
+encrypt_delay(void)
+{
+    if(!havesessionkey ||
+       (I_SUPPORT_ENCRYPT & remote_supports_decrypt) == 0 ||
+       (I_SUPPORT_DECRYPT & remote_supports_encrypt) == 0)
+	return 0;
+    if(!(encrypt_output && decrypt_input))
+	return 1;
+    return 0;
+}
+
 void
 encrypt_debug(int mode)
 {
@@ -959,7 +969,7 @@ void encrypt_gen_printsub(unsigned char *data, int cnt,
     buf[buflen-2] = '*';
     buflen -= 2;;
     for (; cnt > 0; cnt--, data++) {
-	sprintf(tbuf, " %d", *data);
+	snprintf(tbuf, sizeof(tbuf), " %d", *data);
 	for (cp = tbuf; *cp && buflen > 0; --buflen)
 	    *buf++ = *cp++;
 	if (buflen <= 0)
