@@ -33,7 +33,7 @@
 
 #include <config.h>
 
-RCSID("$Id: spx.c,v 1.13 1997/05/20 18:40:41 bg Exp $");
+RCSID("$Id: spx.c,v 1.14 1997/12/05 01:27:16 assar Exp $");
 
 #ifdef	SPX
 /*
@@ -172,7 +172,7 @@ spx_init(ap, server)
 
 	if (server) {
 		str_data[3] = TELQUAL_REPLY;
-		k_gethostname(lhostname, sizeof(lhostname));
+		gethostname(lhostname, sizeof(lhostname));
 		strcpy(targ_printable, "SERVICE:rcmd@");
 		strcat(targ_printable, lhostname);
 		input_name_buffer.length = strlen(targ_printable);
@@ -322,7 +322,7 @@ spx_is(ap, data, cnt)
 		input_token.length = cnt;
 		input_token.value = (char *) data;
 
-		k_gethostname(lhostname, sizeof(lhostname));
+		gethostname(lhostname, sizeof(lhostname));
 
 		strcpy(targ_printable, "SERVICE:rcmd@");
 		strcat(targ_printable, lhostname);
