@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: commands.c,v 1.41 1997/10/29 01:26:34 assar Exp $");
+RCSID("$Id: commands.c,v 1.45 1998/05/01 05:16:55 assar Exp $");
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 int tos = -1;
@@ -2029,7 +2029,7 @@ int
 tn(int argc, char **argv)
 {
     struct hostent *host = 0;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
     struct sockaddr_in6 sin6;
 #endif
     struct sockaddr_in sin;
@@ -2038,7 +2038,7 @@ tn(int argc, char **argv)
     struct servent *sp = 0;
     unsigned long temp;
     extern char *inet_ntoa();
-#if	defined(IP_OPTIONS) && defined(IPPROTO_IP)
+#if defined(IP_OPTIONS) && defined(IPPROTO_IP)
     char *srp = 0;
     int srlen;
 #endif
@@ -2104,11 +2104,7 @@ tn(int argc, char **argv)
 	srp = 0;
 	temp = sourceroute(hostp, &srp, &srlen);
 	if (temp == 0) {
-#ifdef HAVE_H_ERRNO
 	    fprintf (stderr, "%s: %s\r\n", srp ? srp : "", hstrerror(h_errno));
-#else
-	    fprintf (stderr, "%s: %s\r\n", srp ? srp : "", "unknown error");
-#endif
 	    setuid(getuid());
 	    return 0;
 	} else if (temp == -1) {
@@ -2121,17 +2117,21 @@ tn(int argc, char **argv)
     } else {
 #endif
 	memset (&sin, 0, sizeof(sin));
-#if defined(HAVE_INET_PTON) && defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
 	memset (&sin6, 0, sizeof(sin6));
 
 	if(inet_pton(AF_INET6, hostp, &sin6.sin6_addr)) {
 	    sin6.sin6_family = family = AF_INET6;
+	    sa = (struct sockaddr *)&sin6;
+	    sa_size = sizeof(sin6);
 	    strcpy(_hostname, hostp);
 	    hostname =_hostname;
 	} else
 #endif
 	if(inet_aton(hostp, &sin.sin_addr)){
 	    sin.sin_family = family = AF_INET;
+	    sa = (struct sockaddr *)&sin;
+	    sa_size = sizeof(sin);
 	    strcpy(_hostname, hostp);
 	    hostname = _hostname;
 	} else {
@@ -2154,7 +2154,7 @@ tn(int argc, char **argv)
 		    sin.sin_family = family;
 		    sin.sin_addr   = *((struct in_addr *)(*host->h_addr_list));
 		    break;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
 		case AF_INET6:
 		    memset(&sin6, 0, sizeof(sin6));
 		    sa_size = sizeof(sin6);
@@ -2172,11 +2172,7 @@ tn(int argc, char **argv)
 		hostname = _hostname;
 	    } else {
 	        fprintf (stderr, "%s: %s\r\n", hostp ? hostp : "",
-#ifdef HAVE_H_ERRNO
 			 hstrerror(h_errno));
-#else
-			 "unknown error");
-#endif
 		setuid(getuid());
 		return 0;
 	    }
@@ -2215,34 +2211,35 @@ tn(int argc, char **argv)
 	}
 	telnetport = 1;
     }
-    switch(family) {
-    case AF_INET:
-	sin.sin_port = port;
-	printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
-	break;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
-    case AF_INET6: {
+    do {
+	switch(family) {
+	case AF_INET:
+	    sin.sin_port = port;
+	    printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
+	    break;
+#ifdef HAVE_IPV6
+	case AF_INET6: {
 #ifndef INET6_ADDRSTRLEN
 #define INET6_ADDRSTRLEN 46 
 #endif
 
-	char buf[INET6_ADDRSTRLEN];
+	    char buf[INET6_ADDRSTRLEN];
 
-	sin6.sin6_port = port;
+	    sin6.sin6_port = port;
 #ifdef HAVE_INET_NTOP
-	printf("Trying %s...\r\n", inet_ntop(AF_INET6,
-					     &sin6.sin6_addr,
-					     buf,
-					     sizeof(buf)));
+	    printf("Trying %s...\r\n", inet_ntop(AF_INET6,
+						 &sin6.sin6_addr,
+						 buf,
+						 sizeof(buf)));
 #endif
-	break;
-    }
+	    break;
+	}
 #endif
-    default:
-	abort();
-    }
+	default:
+	    abort();
+	}
+
 
-    do {
 	net = socket(family, SOCK_STREAM, 0);
 	setuid(getuid());
 	if (net < 0) {
@@ -2285,7 +2282,7 @@ tn(int argc, char **argv)
 			    inet_ntoa(sin.sin_addr));
 		    sin.sin_addr = *((struct in_addr *)(*++host->h_addr_list));
 		    break;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
 		case AF_INET6: {
 		    char buf[INET6_ADDRSTRLEN];
 
