@@ -38,7 +38,7 @@ static char copyright[] =
 #endif /* not lint */
 
 #include "telnet_locl.h"
-RCSID("$Id: main.c,v 1.21 1997/11/29 02:28:36 joda Exp $");
+RCSID("$Id: main.c,v 1.25 1998/04/04 17:55:54 assar Exp $");
 
 /* These values need to be the same as defined in libtelnet/kerberos5.c */
 /* Either define them in both places, or put in some common header file. */
@@ -90,6 +90,20 @@ usage(void)
  */
 
 
+#ifdef KRB5
+/* XXX ugly hack to setup dns-proxy stuff */
+#define Authenticator asn1_Authenticator
+#include <krb5.h>
+static void
+krb5_init(void)
+{
+    krb5_context context;
+    krb5_init_context(&context);
+    krb5_free_context(context);
+    
+}
+#endif
+
 int
 main(int argc, char **argv)
 {
@@ -99,6 +113,10 @@ main(int argc, char **argv)
 	extern int forward_flags;
 #endif	/* FORWARD */
 
+#ifdef KRB5
+	krb5_init();
+#endif
+	
 	tninit();		/* Clear out things */
 
 	TerminalSaveState();
@@ -119,7 +137,7 @@ main(int argc, char **argv)
 	 */
 	autologin = -1;
 
-	while((ch = getopt(argc, argv, "78DEKLS:X:acde:fFk:l:n:rx")) != EOF) {
+	while((ch = getopt(argc, argv, "78DEKLS:X:abcde:fFk:l:n:rx")) != EOF) {
 		switch(ch) {
 		case '8':
 			eight = 3;	/* binary output and input */
