@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: sys_bsd.c,v 1.19 1997/05/04 09:53:14 assar Exp $");
+RCSID("$Id: sys_bsd.c,v 1.22 1998/05/27 02:19:14 assar Exp $");
 
 /*
  * The following routines try to encapsulate what is system dependent
@@ -440,10 +440,13 @@ TerminalNewMode(int f)
 		tmp_tc.c_iflag &= ~ISTRIP;
 	else
 		tmp_tc.c_iflag |= ISTRIP;
-	if (f & MODE_OUTBIN) {
+	if ((f & MODE_OUTBIN) || (f & MODE_OUT8)) {
 		tmp_tc.c_cflag &= ~(CSIZE|PARENB);
 		tmp_tc.c_cflag |= CS8;
-		tmp_tc.c_oflag &= ~OPOST;
+		if(f & MODE_OUTBIN)
+		    tmp_tc.c_oflag &= ~OPOST;
+		else
+		    tmp_tc.c_oflag |= OPOST;
 	} else {
 		tmp_tc.c_cflag &= ~(CSIZE|PARENB);
 		tmp_tc.c_cflag |= old_tc.c_cflag & (CSIZE|PARENB);
@@ -787,9 +790,6 @@ process_rings(int netin,
     if (netex) {
 	FD_SET(net, &xbits);
     }
-#endif
-#ifdef __CYGWIN32__
-    FD_SET(tout, &obits);
 #endif
     if ((c = select(16, &ibits, &obits, &xbits,
 			(poll == 0)? (struct timeval *)0 : &TimeValue)) < 0) {
@@ -871,7 +871,7 @@ process_rings(int netin,
 		if ((c == -1) && (errno == EINVAL)) {
 		    c = recv(net, netiring.supply, canread, 0);
 		    if (clocks.didnetreceive < clocks.gotDM) {
-			SYNCHing = stilloob(net);
+			SYNCHing = stilloob();
 		    }
 		} else if (first && c > 0) {
 		    /*
