@@ -33,7 +33,7 @@
  *	@(#)ext.h	8.2 (Berkeley) 12/15/93
  */
 
-/* $Id: ext.h,v 1.13 1997/05/20 22:07:49 assar Exp $ */
+/* $Id: ext.h,v 1.15 1997/10/14 20:57:05 joda Exp $ */
 
 #ifndef __EXT_H__
 #define __EXT_H__
@@ -126,7 +126,7 @@ int getterminaltype (char *name);
 void _gettermname (void);
 int terminaltypeok (char *s);
 void doit (struct sockaddr_in *who);
-void telnet (int f, int p);
+void telnet (int f, int p, char*, int, char*);
 void interrupt (void);
 void sendbrk (void);
 void sendsusp (void);
@@ -142,7 +142,7 @@ void netclear (void);
 void netflush (void);
 void writenet (unsigned char *ptr, int len);
 void fatal (int f, char *msg);
-void fatalperror (int f, char *msg);
+void fatalperror (int f, const char *msg);
 void edithost (char *pat, char *host);
 void putstr (char *s);
 void putchr (int cc);
