@@ -33,7 +33,7 @@
 
 #include "telnetd.h"
 
-RCSID("$Id: sys_term.c,v 1.73 1997/05/25 17:08:48 joda Exp $");
+RCSID("$Id: sys_term.c,v 1.79 1997/10/10 22:07:24 assar Exp $");
 
 #if defined(_CRAY) || (defined(__hpux) && !defined(HAVE_UTMPX_H))
 # define PARENT_DOES_UTMP
@@ -1083,6 +1083,22 @@ clean_ttyname (char *tty)
   return res;
 }
 
+/*
+ * Generate a name usable as an `ut_id', typically without `tty'.
+ */
+
+static char *
+make_id (char *tty)
+{
+  char *res = tty;
+  
+  if (strncmp (res, "pts/", 4) == 0)
+    res += 4;
+  if (strncmp (res, "tty", 3) == 0)
+    res += 3;
+  return res;
+}
+
 /*
  * startslave(host)
  *
@@ -1225,9 +1241,11 @@ static int addarg(struct arg_val*, char*);
  * function will turn us into the login process.
  */
 
-void start_login(char *host, int autologin, char *name)
+void
+start_login(char *host, int autologin, char *name)
 {
     struct arg_val argv;
+    char *user;
 
 #ifdef HAVE_UTMPX_H
     char id_buf[3];
@@ -1235,15 +1253,19 @@ void start_login(char *host, int autologin, char *name)
     int pid = getpid();
     struct utmpx utmpx;
     struct timeval tmp;
+    char *clean_tty;
 
     /*
      * Create utmp entry for child
      */
 
+    clean_tty = clean_ttyname(line);
     memset(&utmpx, 0, sizeof(utmpx));
     strncpy(utmpx.ut_user,  ".telnet", sizeof(utmpx.ut_user));
-
-    strncpy(utmpx.ut_line,  clean_ttyname(line), sizeof(utmpx.ut_line));
+    strncpy(utmpx.ut_line,  clean_tty, sizeof(utmpx.ut_line));
+#ifdef HAVE_UT_ID
+    strncpy(utmpx.ut_id, make_id(clean_tty), sizeof(utmpx.ut_id));
+#endif
     utmpx.ut_pid = pid;
 	
     utmpx.ut_type = LOGIN_PROCESS;
@@ -1272,6 +1294,10 @@ void start_login(char *host, int autologin, char *name)
     addarg(&argv, "-h");
     addarg(&argv, host);
     addarg(&argv, "-p");
+    if(name[0])
+	user = name;
+    else
+	user = getenv("USER");
 #ifdef AUTHENTICATION
     if (auth_level < 0 || autologin != AUTH_VALID) {
 	if(!no_warn)
@@ -1281,24 +1307,18 @@ void start_login(char *host, int autologin, char *name)
 	    addarg(&argv, "-a");
 	    addarg(&argv, "otp");
 	}
-	if(log_unauth) {
-	    char *u;
-	    if(name[0]) u = name;
-	    else u=getenv("USER");
-	    u = u ? u : "unknown user";
+	if(log_unauth) 
 	    syslog(LOG_INFO, "unauthenticated access from %s (%s)", 
-		   host, u);
-	}
+		   host, user ? user : "unknown user");
     }
-    if (auth_level >= 0 && autologin == AUTH_VALID) {
+    if (auth_level >= 0 && autologin == AUTH_VALID)
 	addarg(&argv, "-f");
-	addarg(&argv, "--");
-	addarg(&argv, name);
-    } /* else */ /* esc@magic.fi; removed stupid else */
 #endif
-    if (getenv("USER")) {
+    if(user){
 	addarg(&argv, "--");
-	addarg(&argv, strdup(getenv("USER")));
+	addarg(&argv, strdup(user));
+    }
+    if (getenv("USER")) {
 	/*
 	 * Assume that login will set the USER variable
 	 * correctly.  For SysV systems, this means that
@@ -1360,6 +1380,7 @@ rmut(void)
     struct stat statbf;
     struct timeval tmp;
     struct utmpx *utxp, utmpx;
+    char *clean_tty = clean_ttyname(line);
 
     /*
      * This updates the utmpx and utmp entries and make a wtmp/x entry
@@ -1367,12 +1388,13 @@ rmut(void)
 
     setutxent();
     memset(&utmpx, 0, sizeof(utmpx));
-    strncpy(utmpx.ut_line, clean_ttyname(line), sizeof(utmpx.ut_line));
+    strncpy(utmpx.ut_line, clean_tty, sizeof(utmpx.ut_line));
     utmpx.ut_type = LOGIN_PROCESS;
     utxp = getutxline(&utmpx);
     if (utxp) {
 	strcpy(utxp->ut_user, "");
 	utxp->ut_type = DEAD_PROCESS;
+#ifdef HAVE_UT_EXIT
 #ifdef _STRUCT___EXIT_STATUS
 	utxp->ut_exit.__e_termination = 0;
 	utxp->ut_exit.__e_exit = 0;
@@ -1382,6 +1404,7 @@ rmut(void)
 #else	
 	utxp->ut_exit.e_termination = 0;
 	utxp->ut_exit.e_exit = 0;
+#endif
 #endif
 	gettimeofday(&utxp->ut_tv, NULL);
 	pututxline(utxp);
@@ -1393,7 +1416,7 @@ rmut(void)
 	  int f = open(wtmpf, O_WRONLY|O_APPEND);
 	  struct utmp wtmp;
 	  if (f >= 0) {
-	    strncpy(wtmp.ut_line,  clean_ttyname(line), sizeof(wtmp.ut_line));
+	    strncpy(wtmp.ut_line,  clean_tty, sizeof(wtmp.ut_line));
 	    strncpy(wtmp.ut_name,  "", sizeof(wtmp.ut_name));
 #ifdef HAVE_UT_HOST
 	    strncpy(wtmp.ut_host,  "", sizeof(wtmp.ut_host));
@@ -1420,6 +1443,7 @@ rmut(void)
     struct utmp *u, *utmp;
     int nutmp;
     struct stat statbf;
+    char *clean_tty = clean_ttyname(line);
 
     f = open(utmpf, O_RDWR);
     if (f >= 0) {
@@ -1433,7 +1457,7 @@ rmut(void)
 
 	    for (u = utmp ; u < &utmp[nutmp] ; u++) {
 		if (strncmp(u->ut_line,
-			    clean_ttyname(line),
+			    clean_tty,
 			    sizeof(u->ut_line)) ||
 		    u->ut_name[0]==0)
 		    continue;
@@ -1452,7 +1476,7 @@ rmut(void)
     if (found) {
 	f = open(wtmpf, O_WRONLY|O_APPEND);
 	if (f >= 0) {
-	    strncpy(wtmp.ut_line,  clean_ttyname(line), sizeof(wtmp.ut_line));
+	    strncpy(wtmp.ut_line,  clean_tty, sizeof(wtmp.ut_line));
 	    strncpy(wtmp.ut_name,  "", sizeof(wtmp.ut_name));
 #ifdef HAVE_UT_HOST
 	    strncpy(wtmp.ut_host,  "", sizeof(wtmp.ut_host));
