@@ -33,7 +33,7 @@
 
 #include "telnetd.h"
 
-RCSID("$Id: telnetd.c,v 1.41 1997/05/20 13:47:59 assar Exp $");
+RCSID("$Id: telnetd.c,v 1.49 1997/11/14 19:35:13 joda Exp $");
 
 #ifdef _SC_CRAY_SECURE_SYS
 #include <sys/sysv.h>
@@ -124,24 +124,17 @@ extern void usage (void);
  * that only the actual options that we support will be
  * passed off to getopt().
  */
-char valid_opts[] = {
-    'B', 'd', ':', 'h', 'k', 'l', 'n', 'S', ':', 'u', ':', 'U',
-#ifdef	AUTHENTICATION
-    'a', ':', 'X', ':',
+char valid_opts[] = "Bd:hklnS:u:UL:y"
+#ifdef AUTHENTICATION
+		    "a:X:z"
 #endif
 #ifdef DIAGNOSTICS
-    'D', ':',
+		    "D:"
 #endif
 #ifdef _CRAY
-    'r', ':',
+		    "r:"
 #endif
-    'L', ':',
-#ifdef AUTHENTICATION
-    'y',
-    'z',
-#endif
-    '\0'
-};
+		    ;
 
 void doit(struct sockaddr_in*);
 
@@ -308,9 +301,11 @@ int main(int argc, char **argv)
 	     */
 	    auth_disable_name(optarg);
 	    break;
+#endif
 	case 'y':
 	    no_warn = 1;
 	    break;
+#ifdef AUTHENTICATION
 	case 'z':
 	    log_unauth = 1;
 	    break;
@@ -340,13 +335,17 @@ int main(int argc, char **argv)
 	if (argc > 1) {
 	    usage ();
 	} else if (argc == 1) {
-	    sp = getservbyname (*argv, "tcp");
+	    sp = roken_getservbyname (*argv, "tcp");
 	    if (sp)
 		port = sp->s_port;
 	    else
 		port = htons(atoi(*argv));
 	} else {
-	    port = k_getportbyname ("telnet", "tcp", htons(23));
+#ifdef KRB5
+	    port = krb5_getportbyname (NULL, "telnet", "tcp", 23);
+#else
+	    port = k_getportbyname("telnet", "tcp", htons(23));
+#endif
 	}
 	mini_inetd (port);
     } else if (argc > 0) {
@@ -409,7 +408,7 @@ int main(int argc, char **argv)
     }
 #endif	/* _SC_CRAY_SECURE_SYS */
 
-    openlog("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
+    roken_openlog("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
     fromlen = sizeof (from);
     if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0) {
 	fprintf(stderr, "%s: ", progname);
@@ -648,21 +647,7 @@ _gettermname()
 int
 terminaltypeok(char *s)
 {
-    char buf[1024];
-
-    if (terminaltype == NULL)
-	return(1);
-
-    /*
-     * tgetent() will return 1 if the type is known, and
-     * 0 if it is not known.  If it returns -1, it couldn't
-     * open the database.  But if we can't open the database,
-     * it won't help to say we failed, because we won't be
-     * able to verify anything else.  So, we treat -1 like 1.
-     */
-    if (tgetent(buf, s) == 0)
-	return(0);
-    return(1);
+    return 1;
 }
 
 
@@ -670,8 +655,6 @@ char *hostname;
 char host_name[MaxHostNameLen];
 char remote_host_name[MaxHostNameLen];
 
-extern void telnet (int, int);
-
 /*
  * Get a pty, scan input lines.
  */
@@ -708,7 +691,8 @@ doit(struct sockaddr_in *who)
 #endif	/* _SC_CRAY_SECURE_SYS */
 
     /* get name of connected client */
-    hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
+    hp = roken_gethostbyaddr((const char *)&who->sin_addr,
+		       sizeof (struct in_addr),
 		       who->sin_family);
 
     if (hp == NULL && registerd_host_only) {
@@ -727,7 +711,8 @@ Please contact your net administrator");
     remote_host_name[sizeof(remote_host_name)-1] = 0;
     host = remote_host_name;
 
-    k_gethostname(host_name, sizeof (host_name));
+    /* XXX - should be k_gethostname? */
+    gethostname(host_name, sizeof (host_name));
     hostname = host_name;
 
 #ifndef abs
@@ -763,11 +748,6 @@ Please contact your net administrator");
     level = getterminaltype(user_name);
     setenv("TERM", terminaltype ? terminaltype : "network", 1);
 
-    /*
-     * Start up the login process on the slave side of the terminal
-     */
-    startslave(host, level, user_name);
-
 #ifdef _SC_CRAY_SECURE_SYS
     if (secflag) {
 	if (setulvl(dv.dv_actlvl) < 0)
@@ -777,27 +757,47 @@ Please contact your net administrator");
     }
 #endif	/* _SC_CRAY_SECURE_SYS */
 
-    telnet(net, ourpty);  /* begin server processing */
+    telnet(net, ourpty, host, level, user_name);  /* begin server processing */
     /*NOTREACHED*/
 }  /* end of doit */
 
+/* output contents of /etc/issue.net, or /etc/issue */
+static void
+show_issue(void)
+{
+    FILE *f;
+    char buf[128];
+    f = fopen("/etc/issue.net", "r");
+    if(f == NULL)
+	f = fopen("/etc/issue", "r");
+    if(f){
+	while(fgets(buf, sizeof(buf)-2, f)){
+	    strcpy(buf + strcspn(buf, "\r\n"), "\r\n");
+	    writenet((unsigned char*)buf, strlen(buf));
+	}
+	fclose(f);
+    }
+}
+
 /*
  * Main loop.  Select from pty and network, and
  * hand data to telnet receiver finite state machine.
  */
 void
-telnet(int f, int p)
+telnet(int f, int p, char *host, int level, char *autoname)
 {
     int on = 1;
 #define	TABBUFSIZ	512
     char	defent[TABBUFSIZ];
     char	defstrs[TABBUFSIZ];
 #undef	TABBUFSIZ
-    char *HE;
+    char *he;
     char *HN;
     char *IM;
     void netflush();
     int nfd;
+    int startslave_called = 0;
+    time_t timeout;
 
     /*
      * Initialize the slc mapping table.
@@ -935,6 +935,7 @@ telnet(int f, int p)
     }
 #endif
 
+    show_issue();
     /*
      * Show banner that getty never gave.
      *
@@ -947,8 +948,8 @@ telnet(int f, int p)
 	hostinfo = 0;
 
     IM = DEFAULT_IM;
-    HE = 0;
-    edithost(HE, host_name);
+    he = 0;
+    edithost(he, host_name);
     if (hostinfo && *IM)
 	putf(IM, ptyibuf2);
 
@@ -963,10 +964,18 @@ telnet(int f, int p)
 
 
     nfd = ((f > p) ? f : p) + 1;
+    timeout = time(NULL) + 5;
     for (;;) {
 	fd_set ibits, obits, xbits;
 	int c;
 
+	/* wait for encryption to be turned on, but don't wait
+           indefinitely */
+	if(!startslave_called && (!encrypt_delay() || timeout > time(NULL))){
+	    startslave_called = 1;
+	    startslave(host, level, autoname);
+	}
+
 	if (ncc < 0 && pcc < 0)
 	    break;
 
