@@ -8,7 +8,7 @@
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: xnlock.c,v 1.64 1997/11/29 03:58:13 joda Exp $");
+RCSID("$Id: xnlock.c,v 1.71 1998/04/22 01:33:30 joda Exp $");
 #endif
 
 #include <stdio.h>
@@ -20,6 +20,9 @@ RCSID("$Id: xnlock.c,v 1.64 1997/11/29 03:58:13 joda Exp $");
 #include <X11/keysym.h>
 #include <X11/Shell.h>
 #include <X11/Xos.h>
+#ifdef strerror
+#undef strerror
+#endif
 #include <ctype.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -33,8 +36,6 @@ RCSID("$Id: xnlock.c,v 1.64 1997/11/29 03:58:13 joda Exp $");
 #endif
 #ifdef KRB4
 #include <krb.h>
-#endif
-#if defined(KRB4)
 #include <kafs.h>
 #endif
 
@@ -166,25 +167,28 @@ static char*
 get_words(void)
 {
     FILE *pp = NULL;
-    static char buf[BUFSIZ];
+    static char buf[512];
+    long n;
 
-    if(appres.text_prog){
+    if (appres.text_prog) {
 	pp = popen(appres.text_prog, "r");
-	if(!pp){
-	    warn ("popen %s", appres.text_prog);
+	if (!pp) {
+	    warn("popen %s", appres.text_prog);
 	    return appres.text;
 	}
-	fread(buf, BUFSIZ, 1, pp);
+	n = fread(buf, 1, sizeof(buf) - 1, pp);
+	buf[n] = 0;
 	pclose(pp);
 	return buf;
     }
-    if(appres.file){
+    if (appres.file) {
 	pp = fopen(appres.file, "r");
-	if(!pp){
-	    warn ("fopen %s", appres.file);
+	if (!pp) {
+	    warn("fopen %s", appres.file);
 	    return appres.text;
 	}
-	fread(buf, BUFSIZ, 1, pp);
+	n = fread(buf, 1, sizeof(buf) - 1, pp);
+	buf[n] = 0;
 	fclose(pp);
 	return buf;
     }
@@ -216,7 +220,8 @@ init_words (int argc, char **argv)
     int i = 0;
 
     while(argv[i]) {
-	if(strcmp(argv[i], "-p") == 0) {
+	if(strcmp(argv[i], "-p") == 0
+	   || strcmp(argv[i], "-prog") == 0) {
 	    i++;
 	    if(argv[i]) {
 		appres.text_prog = argv[i];
@@ -1059,10 +1064,14 @@ main (int argc, char **argv)
     }
 
     XtRealizeWidget(override);
-    XGrabPointer(dpy, XtWindow(widget), TRUE, 0, GrabModeAsync,
-		 GrabModeAsync, XtWindow(widget), None, CurrentTime);
-    XGrabKeyboard(dpy, XtWindow(widget), TRUE, GrabModeAsync,
-		  GrabModeAsync, CurrentTime);
+    if((i = XGrabPointer(dpy, XtWindow(widget), True, 0, GrabModeAsync,
+			 GrabModeAsync, XtWindow(widget), 
+			 None, CurrentTime)) != 0) 
+	errx(1, "Failed to grab pointer (%d)", i);
+	
+    if((i = XGrabKeyboard(dpy, XtWindow(widget), True, GrabModeAsync,
+			  GrabModeAsync, CurrentTime)) != 0)
+	errx(1, "Failed to grab keyboard (%d)", i);
     ScreenSaver(1);
     XtAppMainLoop(app);
     exit(0);
