@@ -30,7 +30,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Id: admin_server.c,v 1.40 1997/05/18 20:02:07 assar Exp $");
+RCSID("$Id: admin_server.c,v 1.43 1998/05/11 09:19:41 joda Exp $");
 
 /* Almost all procs and such need this, so it is global */
 admin_params prm;		/* The command line parameters struct */
@@ -262,6 +262,8 @@ kadm_listen(void)
     signal(SIGPIPE, SIG_IGN); /* get errors on write() */
     signal(SIGALRM, doexit);
     signal(SIGCHLD, do_child);
+    if (setsid() < 0)
+        krb_log("setsid() failed");
 
     if ((admin_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
 	return KADM_NO_SOCK;
@@ -313,13 +315,9 @@ kadm_listen(void)
 		}
 		/* fork succeded: keep tabs on child */
 		close(peer_fd);
-		if (pidarray) {
-		    pidarray = (int *)realloc(pidarray, ++pidarraysize);
-		    pidarray[pidarraysize-1] = pid;
-		} else {
-		    pidarray = (int *)malloc(pidarraysize = 1);
-		    pidarray[0] = pid;
-		}
+		pidarray = realloc(pidarray, 
+				   (pidarraysize + 1) * sizeof(*pidarray));
+		pidarray[pidarraysize++] = pid;
 	    } else {
 		/* child */
 		close(admin_fd);
