@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -36,15 +36,11 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kadm_locl.h,v 1.20 1996/11/17 23:41:28 assar Exp $ */
+/* $Id: kadm_locl.h,v 1.26 1997/09/02 18:13:26 assar Exp $ */
 
 #include "config.h"
 #include "protos.h"
 
-#ifdef SOCKS
-#include <socks.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -57,13 +53,11 @@
 #ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
-#else
-#ifdef HAVE_SYS_TIME_H
+#elif defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
 #else
 #include <time.h>
-#endif /* !HAVE_SYS_TIME_H */
-#endif /* !TIME_WITH_SYS_TIME */
+#endif
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
@@ -105,6 +99,12 @@
 #include <syslog.h>
 #endif
 
+#include <err.h>
+
+#ifdef SOCKS
+#include <socks.h>
+#endif
+
 #include <roken.h>
 
 #include <com_err.h>
@@ -118,6 +118,8 @@
 #include <kadm_err.h>
 #include <acl.h>
 
+#include <krb_log.h>
+
 #include "kadm_server.h"
 #include "pw_check.h"
 
@@ -142,4 +144,7 @@ int kadm_ser_delete __P((u_char *, int, AUTH_DAT *, u_char **, int *));
 int kadm_ser_init __P((int inter, char realm[]));
 int kadm_ser_in __P((u_char **, int *));
 
-int get_pw_new_pwd(char *pword, int pwlen, krb_principal *pr, int print_realm);
+int get_pw_new_pwd  __P((char *pword, int pwlen, krb_principal *pr, int print_realm));
+
+/* cracklib */
+char *FascistCheck __P((char *password, char *path, char **strings));
