@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -35,7 +30,7 @@ unwraps wrapped packets and calls the appropriate server subroutine
 
 #include "kadm_locl.h"
 
-RCSID("$Id: kadm_ser_wrap.c,v 1.19 1996/11/19 22:48:42 assar Exp $");
+RCSID("$Id: kadm_ser_wrap.c,v 1.21 1997/12/05 01:27:29 assar Exp $");
 
 /* GLOBAL */
 Kadm_Server server_parm;
@@ -54,7 +49,7 @@ kadm_ser_init(int inter, char *realm)
 
   init_kadm_err_tbl();
   init_krb_err_tbl();
-  if (k_gethostname(hostname, sizeof(hostname)))
+  if (gethostname(hostname, sizeof(hostname)))
       return KADM_NO_HOSTNAME;
 
   strcpy(server_parm.sname, PWSERV_NAME);
