@@ -1,10 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- * $Id: kadm_server.h,v 1.8 1996/06/12 21:28:40 bg Exp $
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,6 +19,8 @@ or implied warranty.
 
   */
 
+/* $Id: kadm_server.h,v 1.10 1997/05/11 04:08:26 assar Exp $ */
+
 /*
  * Definitions for Kerberos administration server & client
  */
@@ -41,9 +36,6 @@ or implied warranty.
  *        as part of our ongoing attempt to copyright all greek names
  */
 
-#include <krb.h>
-#include <des.h>
-
 typedef struct {
   struct sockaddr_in admin_addr;
   struct sockaddr_in recv_addr;
