@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -34,18 +29,18 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Id: kadmin.c,v 1.42 1996/11/23 23:25:37 assar Exp $");
+RCSID("$Id: kadmin.c,v 1.51 1998/05/11 20:56:56 joda Exp $");
 
-static void change_password(int argc, char **argv);
-static void change_key(int argc, char **argv);
-static void change_admin_password(int argc, char **argv);
-static void add_new_key(int argc, char **argv);
-static void del_entry(int argc, char **argv);
-static void get_entry(int argc, char **argv);
-static void mod_entry(int argc, char **argv);
-static void help(int argc, char **argv);
-static void clean_up_cmd(int argc, char **argv);
-static void quit_cmd(int argc, char **argv);
+static int change_password(int argc, char **argv);
+static int change_key(int argc, char **argv);
+static int change_admin_password(int argc, char **argv);
+static int add_new_key(int argc, char **argv);
+static int del_entry(int argc, char **argv);
+static int get_entry(int argc, char **argv);
+static int mod_entry(int argc, char **argv);
+static int help(int argc, char **argv);
+static int clean_up_cmd(int argc, char **argv);
+static int quit_cmd(int argc, char **argv);
 
 static SL_cmd cmds[] = {
   {"change_password", change_password, "Change a user's password"},
@@ -177,9 +172,11 @@ princ_exists(char *name, char *instance, char *realm)
 {
     int status;
 
+    int old = krb_use_admin_server(1);
     status = krb_get_pw_in_tkt(name, instance, realm,
 			       KRB_TICKET_GRANTING_TICKET,
 			       realm, 1, "");
+    krb_use_admin_server(old);
 
     if ((status == KSUCCESS) || (status == INTK_BADPW))
 	return(PE_YES);
@@ -252,9 +249,12 @@ get_admin_password(void)
     }
     
     if (princ_exists(pr.name, pr.instance, pr.realm) != PE_NO) {
-	if (read_long_pw_string(admin_passwd, sizeof(admin_passwd)-1,
-				"Admin password:", 0)) {
-	    fprintf(stderr, "Error reading admin password.\n");
+        char prompt[256];
+	snprintf(prompt, sizeof(prompt), "%s's Password: ", krb_unparse_name(&pr));
+	if (read_long_pw_string(admin_passwd,
+				sizeof(admin_passwd)-1,
+				prompt, 0)) {
+	    warnx ("Error reading admin password.");
 	    goto bad;
 	}
 	status = krb_get_pw_in_tkt(pr.name, pr.instance, pr.realm,
@@ -309,25 +309,17 @@ clean_up()
     dest_tkt();
 }
 
-static void
+static int
 clean_up_cmd (int argc, char **argv)
 {
     clean_up();
+    return 0;
 }
 
-/* GLOBAL */
-static void 
-quit()
-{
-    printf("Cleaning up and exiting.\n");
-    clean_up();
-    exit(0);
-}
-
-static void
+static int
 quit_cmd (int argc, char **argv)
 {
-    quit();
+    return 1;
 }
 
 static void
@@ -338,21 +330,17 @@ do_init(int argc, char **argv)
     char tktstring[MaxPathLen];
     int k_errno;
     
+    set_progname (argv[0]);
+
     memset(&pr, 0, sizeof(pr));
-    if (krb_get_default_principal(pr.name, pr.instance, default_realm) < 0) {
-	fprintf (stderr, "%s: I could not even guess who you might be\n",
-		 argv[0]);
-	exit (1);
-    }	
+    if (krb_get_default_principal(pr.name, pr.instance, default_realm) < 0)
+	errx (1, "I could not even guess who you might be");
     while ((c = getopt(argc, argv, "p:u:r:mt")) != EOF) 
 	switch (c) {
 	case 'p':
 	case 'u':
-	    if((k_errno = krb_parse_name(optarg, &pr)) != KSUCCESS) {
-		fprintf (stderr, "%s: %s\n", argv[0],
-			 krb_get_err_text(k_errno));
-		exit (1);
-	    }
+	    if((k_errno = krb_parse_name(optarg, &pr)) != KSUCCESS)
+		errx (1, "%s", krb_get_err_text(k_errno));
 	    break;
 	case 'r':
 	    memset(default_realm, 0, sizeof(default_realm));
@@ -380,7 +368,7 @@ do_init(int argc, char **argv)
 	strcpy(pr.instance, "admin");
     
     if (!tflag) {
-	sprintf(tktstring, TKT_ROOT "_adm_%d",(int)getpid());
+	snprintf(tktstring, sizeof(tktstring), TKT_ROOT "_adm_%d",(int)getpid());
 	krb_set_tkt_string(tktstring);
     }
     
@@ -394,8 +382,8 @@ main(int argc, char **argv)
     printf("Welcome to the Kerberos Administration Program, version 2\n");
     printf("Type \"help\" if you need it.\n");
     sl_loop (cmds, "kadmin: ");
-    printf("\n");
-    quit();
+    printf("\nCleaning up and exiting.\n");
+    clean_up();
     exit(0);
 }
 
@@ -429,7 +417,7 @@ setvals(Kadm_vals *vals, char *string)
 	return KADM_SUCCESS;
 }    
 
-static void 
+static int 
 change_password(int argc, char **argv)
 {
     Kadm_vals old, new;
@@ -438,11 +426,11 @@ change_password(int argc, char **argv)
 
     if (argc != 2) {
 	printf("Usage: change_password loginname\n");
-	return;
+	return 0;
     }
 
     if (setvals(&old, argv[1]) != KADM_SUCCESS)
-	return;
+	return 0;
 
     new = old;
 
@@ -451,10 +439,10 @@ change_password(int argc, char **argv)
     if (princ_exists(old.name, old.instance, krbrlm) != PE_NO) {
 	/* get the admin's password */
         if (get_admin_password() != GOOD_PW)
-	    return;
+	    return 0;
 
 	/* get the new password */
-	sprintf(pw_prompt, "New password for %s:", argv[1]);
+	snprintf(pw_prompt, sizeof(pw_prompt), "New password for %s:", argv[1]);
 	
 	if (get_password(&new.key_low, &new.key_high,
 			 pw_prompt, SWAP) == GOOD_PW) {
@@ -474,7 +462,7 @@ change_password(int argc, char **argv)
     else 
 	printf("kadmin: Principal %s does not exist.\n",
 	       krb_unparse_name_long (old.name, old.instance, krbrlm));
-    return;
+    return 0;
 }
 
 static int
@@ -517,7 +505,7 @@ printkey(unsigned char *tkey)
     printf("\n");
 }
 
-static void 
+static int 
 change_key(int argc, char **argv)
 {
     Kadm_vals old, new;
@@ -526,11 +514,11 @@ change_key(int argc, char **argv)
 
     if (argc != 2) {
 	printf("Usage: change_key principal-name\n");
-	return;
+	return 0;
     }
 
     if (setvals(&old, argv[1]) != KADM_SUCCESS)
-	return;
+	return 0;
 
     new = old;
 
@@ -539,7 +527,7 @@ change_key(int argc, char **argv)
     if (princ_exists(old.name, old.instance, krbrlm) != PE_NO) {
 	/* get the admin's password */
         if (get_admin_password() != GOOD_PW)
-	    return;
+	    return 0;
 
 	/* get the new password */
 	printf("New DES key for %s: ", argv[1]);
@@ -567,10 +555,10 @@ change_key(int argc, char **argv)
     else 
 	printf("kadmin: Principal %s does not exist.\n",
 	       krb_unparse_name_long (old.name, old.instance, krbrlm));
-    return;
+    return 0;
 }
 
-static void 
+static int 
 change_admin_password(int argc, char **argv)
 {
     des_cblock newkey;
@@ -580,7 +568,7 @@ change_admin_password(int argc, char **argv)
 
     if (argc != 1) {
 	printf("Usage: change_admin_password\n");
-	return;
+	return 0;
     }
     if (get_pw_new_pwd(pword, sizeof(pword), &pr, 1) == 0) {
 	 des_string_to_key(pword, &newkey);
@@ -596,10 +584,10 @@ change_admin_password(int argc, char **argv)
     }
     if (!multiple)
 	clean_up();
-    return;
+    return 0;
 }
 
-static void 
+static int 
 add_new_key(int argc, char **argv)
 {
     Kadm_vals new;
@@ -608,10 +596,10 @@ add_new_key(int argc, char **argv)
 
     if (argc != 2) {
 	printf("Usage: add_new_key user_name.\n");
-	return;
+	return 0;
     }
     if (setvals(&new, argv[1]) != KADM_SUCCESS)
-	return;
+	return 0;
 
     SET_FIELD(KADM_EXPDATE,new.fields);
     SET_FIELD(KADM_ATTR,new.fields);
@@ -625,7 +613,7 @@ add_new_key(int argc, char **argv)
 
 	/* get the admin's password */
 	if (get_admin_password() != GOOD_PW)
-	    return;
+	    return 0;
 	
 	memset(fields, 0, sizeof(fields));
 	SET_FIELD(KADM_NAME,fields);
@@ -633,18 +621,20 @@ add_new_key(int argc, char **argv)
 	SET_FIELD(KADM_EXPDATE,fields);
 	SET_FIELD(KADM_ATTR,fields);
 	SET_FIELD(KADM_MAXLIFE,fields);
-	sprintf (n, "default.%s", new.instance);
+	snprintf (n, sizeof(n), "default.%s", new.instance);
 	if (setvals(&vals, n) != KADM_SUCCESS)
-	    return;
+	    return 0;
 
 	if (kadm_get(&vals, fields) != KADM_SUCCESS) {
 	    if (setvals(&vals, "default") != KADM_SUCCESS)
-		return;
+		return 0;
 	    if ((status = kadm_get(&vals, fields)) != KADM_SUCCESS) {
 		printf ("kadm error: %s\n", error_message(status));
-		return;
+		return 0;
 	    }
-	    if (vals.max_life == 255) /* Defaults not set! */ {
+	}
+
+	if (vals.max_life == 255) /* Defaults not set! */ {
 	      /* This is the default maximum lifetime for new principals. */
 	      if (strcmp(new.instance, "admin") == 0)
 		vals.max_life = 1 + (CLOCK_SKEW/(5*60)); /* 5+5 minutes */
@@ -656,12 +646,13 @@ add_new_key(int argc, char **argv)
 		vals.max_life = 255;     /* ca 21 hours (maximum) */
 
 	      /* Also fix expiration date. */
+	      vals.exp_date = time(NULL);
 	      if (strcmp(new.name, "rcmd") == 0)
-		vals.exp_date = 1104814999; /* Tue Jan 4 06:03:19 2005 */
+		  vals.exp_date += 5*(365*24*60*60); /* + ca 5 years */
 	      else
-		vals.exp_date = time(0) + 2*(365*24*60*60); /* + ca 2 years */
-	    }
+		  vals.exp_date += 2*(365*24*60*60); /* + ca 2 years */
 	}
+
 	new.max_life = vals.max_life;
 	new.exp_date = vals.exp_date;
 	new.attributes = vals.attributes;
@@ -670,7 +661,7 @@ add_new_key(int argc, char **argv)
 	get_expdate(&new);
 
 	/* get the new password */
-	sprintf(pw_prompt, "Password for %s:", argv[1]);
+	snprintf(pw_prompt, sizeof(pw_prompt), "Password for %s:", argv[1]);
 	
 	if (get_password(&new.key_low, &new.key_high,
 			 pw_prompt, SWAP) == GOOD_PW) {
@@ -688,10 +679,10 @@ add_new_key(int argc, char **argv)
     }
     else
 	printf("kadmin: Principal already exists.\n");
-    return;
+    return 0;
 }
 
-static void 
+static int 
 del_entry(int argc, char **argv)
 {
     int status;
@@ -699,16 +690,16 @@ del_entry(int argc, char **argv)
 
     if (argc != 2) {
 	printf("Usage: del_entry username\n");
-	return;
+	return 0;
     }
 
     if (setvals(&vals, argv[1]) != KADM_SUCCESS)
-	return;
+	return 0;
 
     if (princ_exists(vals.name, vals.instance, krbrlm) != PE_NO) {
 	/* get the admin's password */
 	if (get_admin_password() != GOOD_PW)
-	    return;
+	    return 0;
 	
 	if ((status = kadm_del(&vals)) == KADM_SUCCESS){
 	    printf("%s removed from database.\n", argv[1]);
@@ -722,10 +713,10 @@ del_entry(int argc, char **argv)
     else
 	printf("kadmin: Principal %s does not exist.\n",
 	       krb_unparse_name_long (vals.name, vals.instance, krbrlm));
-    return;
+    return 0;
 }
 
-static void 
+static int 
 get_entry(int argc, char **argv)
 {
     int status;
@@ -734,7 +725,7 @@ get_entry(int argc, char **argv)
 
     if (argc != 2) {
 	printf("Usage: get_entry username\n");
-	return;
+	return 0;
     }
 
     memset(fields, 0, sizeof(fields));
@@ -749,13 +740,13 @@ get_entry(int argc, char **argv)
 #endif
 
     if (setvals(&vals, argv[1]) != KADM_SUCCESS)
-	return;
+	return 0;
 
 
     if (princ_exists(vals.name, vals.instance, krbrlm) != PE_NO) {
 	/* get the admin's password */
 	if (get_admin_password() != GOOD_PW)
-	    return;
+	    return 0;
 	
 	if ((status = kadm_get(&vals, fields)) == KADM_SUCCESS)
 	    prin_vals(&vals);
@@ -768,10 +759,10 @@ get_entry(int argc, char **argv)
     else
 	printf("kadmin: Principal %s does not exist.\n",
 	       krb_unparse_name_long (vals.name, vals.instance, krbrlm));
-    return;
+    return 0;
 }
 
-static void 
+static int 
 mod_entry(int argc, char **argv)
 {
     int status;
@@ -780,7 +771,7 @@ mod_entry(int argc, char **argv)
 
     if (argc != 2) {
 	printf("Usage: mod_entry username\n");
-	return;
+	return 0;
     }
 
     memset(fields, 0, sizeof(fields));
@@ -792,19 +783,19 @@ mod_entry(int argc, char **argv)
     SET_FIELD(KADM_MAXLIFE,fields);
 
     if (setvals(&ovals, argv[1]) != KADM_SUCCESS)
-	return;
+	return 0;
 
     nvals = ovals;
 
     if (princ_exists(ovals.name, ovals.instance, krbrlm) == PE_NO) {
 	printf("kadmin: Principal %s does not exist.\n",
 	       krb_unparse_name_long (ovals.name, ovals.instance, krbrlm));
-	return;
+	return 0;
     }
 
     /* get the admin's password */
     if (get_admin_password() != GOOD_PW)
-	return;
+	return 0;
 	
     if ((status = kadm_get(&ovals, fields)) != KADM_SUCCESS) {
 	printf("[ unable to retrieve current settings: %s ]\n",
@@ -839,11 +830,12 @@ mod_entry(int argc, char **argv)
 out:
     if (!multiple)
 	clean_up();
-    return;
+    return 0;
 }
 
-static void
+static int
 help(int argc, char **argv)
 {
     sl_help (cmds, argc, argv);
+    return 0;
 }
