@@ -1,10 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- * $Id: ksrvutil.c,v 1.34 1996/11/24 00:59:18 assar Exp $
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -37,7 +30,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Id: ksrvutil.c,v 1.34 1996/11/24 00:59:18 assar Exp $");
+RCSID("$Id: ksrvutil.c,v 1.43 1998/03/07 22:02:18 joda Exp $");
 
 #include "ksrvutil.h"
 
@@ -55,8 +48,6 @@ RCSID("$Id: ksrvutil.c,v 1.34 1996/11/24 00:59:18 assar Exp $");
 #define PRINC_HEADER "  Principal\n"
 #define PRINC_FORMAT "%s"
 
-char *progname;
-
 char u_name[ANAME_SZ];
 char u_inst[INST_SZ];
 char u_realm[REALM_SZ];
@@ -94,26 +85,17 @@ copy_keyfile(char *keyfile, char *backup_keyfile)
     do {
 	try_again = FALSE;
 	if ((keyfile_fd = open(keyfile, O_RDONLY, 0)) < 0) {
-	    if (errno != ENOENT) {
-		fprintf(stderr, "%s: Unable to read %s: %s\n", progname, 
-			      keyfile, strerror(errno));
-		exit(1);
-	    }
+	    if (errno != ENOENT)
+	      err (1, "read %s", keyfile);
 	    else {
 		try_again = TRUE;
 		if ((keyfile_fd = 
 		     open(keyfile, 
-			  O_WRONLY | O_TRUNC | O_CREAT, SRVTAB_MODE)) < 0) {
-		    fprintf(stderr, "%s: Unable to create %s: %s\n", 
-				   progname, keyfile, strerror(errno));
-		    exit(1);
-		}
+			  O_WRONLY | O_TRUNC | O_CREAT, SRVTAB_MODE)) < 0)
+		  err(1, "create %s", keyfile);
 		else
-		    if (close(keyfile_fd) < 0) {
-			fprintf(stderr, "%s: Failure closing %s: %s\n",
-				       progname, keyfile, strerror(errno));
-			exit(1);
-		    }
+		    if (close(keyfile_fd) < 0)
+		      err (1, "close %s", keyfile);
 	    }
 	}
     } while(try_again);
@@ -122,33 +104,18 @@ copy_keyfile(char *keyfile, char *backup_keyfile)
 
     if ((backup_keyfile_fd = 
 	 open(backup_keyfile, O_WRONLY | O_TRUNC | O_CREAT, 
-	      keyfile_mode)) < 0) {
-	fprintf(stderr, "%s: Unable to write %s: %s\n", progname, 
-		       backup_keyfile, strerror(errno));
-	exit(1);
-    }
+	      keyfile_mode)) < 0)
+	err (1, "write %s", backup_keyfile);
     do {
-	if ((rcount = read(keyfile_fd, buf, sizeof(buf))) < 0) {
-	    fprintf(stderr, "%s: Error reading %s: %s\n", progname,
-			   keyfile, strerror(errno));
-	    exit(1);
-	}
-	if (rcount && (write(backup_keyfile_fd, buf, rcount) != rcount)) {
-	    fprintf(stderr, "%s: Error writing %s: %s\n", progname,
-			   backup_keyfile, strerror(errno));
-	    exit(1);
-	}
+	if ((rcount = read(keyfile_fd, buf, sizeof(buf))) < 0)
+	    err (1, "read %s", keyfile);
+	if (rcount && (write(backup_keyfile_fd, buf, rcount) != rcount))
+	    err (1, "write %s", backup_keyfile);
     } while (rcount);
-    if (close(backup_keyfile_fd) < 0) {
-	fprintf(stderr, "%s: Error closing %s: %s\n", progname,
-		       backup_keyfile, strerror(errno));
-	exit(1);
-    }
-    if (close(keyfile_fd) < 0) {
-	fprintf(stderr, "%s: Error closing %s: %s\n", progname,
-		       keyfile, strerror(errno));
-	exit(1);
-    }
+    if (close(backup_keyfile_fd) < 0)
+	err(1, "close %s", backup_keyfile);
+    if (close(keyfile_fd) < 0)
+	err(1, "close %s", keyfile);
 }
 
 void
@@ -168,8 +135,7 @@ safe_read_stdin(char *prompt, char *buf, size_t size)
     fflush(stdout);
     memset(buf, 0, size);
     if (read(0, buf, size - 1) < 0) {
-	fprintf(stderr, "Failure reading from stdin: %s\n", 
-		       strerror(errno));
+	warn("read stdin");
 	leave(NULL, 1);
     }
     buf[strlen(buf)-1] = 0;
@@ -179,8 +145,7 @@ void
 safe_write(char *filename, int fd, void *buf, size_t len)
 {
     if (write(fd, buf, len) != len) {
-	fprintf(stderr, "%s: Failure writing to %s: %s\n", progname,
-		       filename, strerror(errno));
+	warn("write %s", filename);
 	close(fd);
 	leave("In progress srvtab in this file.", 1);
     }
@@ -294,13 +259,14 @@ static void
 usage(void)
 {
     fprintf(stderr, "Usage: ksrvutil [-f keyfile] [-i] [-k] ");
-    fprintf(stderr, "[-p principal] [-r realm] ");
+    fprintf(stderr, "[-p principal] [-r realm] [-u]");
     fprintf(stderr, "[-c AFS cellname] ");
     fprintf(stderr, "{list | change | add | get}\n");
-    fprintf(stderr, "   -i causes the program to ask for ");
-    fprintf(stderr, "confirmation before changing keys.\n");
-    fprintf(stderr, "   -k causes the key to printed for list or ");
-    fprintf(stderr, "change.\n");
+    fprintf(stderr, "   -i causes the program to ask for "
+	    "confirmation before changing keys.\n");
+    fprintf(stderr, "   -k causes the key to printed for list or change.\n");
+    fprintf(stderr, "   -u creates one keyfile for each principal "
+	    "(only used with `get')\n");
     exit(1);
 }
 
@@ -327,7 +293,9 @@ main(int argc, char **argv)
     int interactive = FALSE;
     int list = FALSE;
     int change = FALSE;
+    int unique_filename = FALSE;
     int add = FALSE;
+    int delete = FALSE;
     int get = FALSE;
     int key = FALSE;		/* do we show keys? */
     int arg_entered = FALSE;
@@ -346,142 +314,139 @@ main(int argc, char **argv)
     memset(local_realm, 0, sizeof(local_realm));
     memset(cellname, 0, sizeof(cellname));
     
-    progname=argv[0];
+    set_progname (argv[0]);
 
-    if (krb_get_default_principal(u_name, u_inst, u_realm) < 0) {
-	 fprintf (stderr, "%s: could not get default principal\n", progname);
-	 return 1;
-    }
+    if (krb_get_default_principal(u_name, u_inst, u_realm) < 0)
+	errx (1, "could not get default principal");
 
     /* This is used only as a default for adding keys */
     if (krb_get_lrealm(local_realm, 1) != KSUCCESS)
 	strcpy(local_realm, KRB_REALM);
     
-    while((c = getopt(argc, argv, "ikc:f:p:r:")) != EOF) {
-	 switch (c) {
-	      case 'i':
-	      interactive++;
-	      break;
-	      case 'k':
-	      key++;
-	      break;
-	      case 'c':
-	      strcpy(cellname, optarg);
-	      break;
-	      case 'f':
-	      strcpy(keyfile, optarg);
-	      break;
-	      case 'p':
-	      if((status = kname_parse (u_name, u_inst, u_realm, optarg)) !=
-		 KSUCCESS) {
-		   fprintf(stderr, "%s: principal %s: %s\n", progname,
-			   optarg,
-			   krb_get_err_text(status));
-		   return 1;
-	      }
-	      break;
-	      case 'r':
-	      strcpy(u_realm, optarg);
-	      break;
-	      case '?':
-	      usage();
-	 }
+    while((c = getopt(argc, argv, "ikc:f:p:r:u")) != EOF) {
+	switch (c) {
+	case 'i':
+	    interactive++;
+	    break;
+	case 'k':
+	    key++;
+	    break;
+	case 'c':
+	    strcpy(cellname, optarg);
+	    break;
+	case 'f':
+	    strcpy(keyfile, optarg);
+	    break;
+	case 'p':
+	    if((status = kname_parse (u_name, u_inst, u_realm, optarg)) !=
+	       KSUCCESS)
+		errx (1, "principal %s: %s", optarg,
+		      krb_get_err_text(status));
+	    break;
+	case 'r':
+	    strcpy(u_realm, optarg);
+	    break;
+	case 'u':
+	    unique_filename = 1;
+	    break;
+	case '?':
+	    usage();
+	}
     }
     if (optind >= argc)
-	 usage();
+	usage();
+    if (*u_realm == '\0')
+	strcpy (u_realm, local_realm);
     if (strcmp(argv[optind], "list") == 0) {
-	 if (arg_entered)
-	      usage();
-	 else {
-	      arg_entered++;
-	      list++;
-	 }
+	if (arg_entered)
+	    usage();
+	else {
+	    arg_entered++;
+	    list++;
+	}
     }
     else if (strcmp(argv[optind], "change") == 0) {
-	 if (arg_entered)
-	      usage();
-	 else {
-	      arg_entered++;
-	      change++;
-	 }
+	if (arg_entered)
+	    usage();
+	else {
+	    arg_entered++;
+	    change++;
+	}
     }
     else if (strcmp(argv[optind], "add") == 0) {
-	 if (arg_entered)
-	      usage();
-	 else {
-	      arg_entered++;
-	      add++;
-	 }
+	if (arg_entered)
+	    usage();
+	else {
+	    arg_entered++;
+	    add++;
+	}
     }
     else if (strcmp(argv[optind], "get") == 0) {
-	 if (arg_entered)
-	      usage();
-	 else {
-	      arg_entered++;
-	      get++;
-	 }
+	if (arg_entered)
+	    usage();
+	else {
+	    arg_entered++;
+	    get++;
+	}
+    }
+    else if (strcmp(argv[optind], "delete") == 0) {
+	if (arg_entered)
+	    usage();
+	else {
+	    arg_entered++;
+	    delete++;
+	}
     }
     else
-	 usage();
+	usage();
     ++optind;
     
     if (!arg_entered)
 	usage();
 
+    if(unique_filename && !get)
+	warnx("`-u' flag is only used with `get'");
+
     if (!keyfile[0])
 	strcpy(keyfile, KEYFILE);
     
     strcpy(work_keyfile, keyfile);
     strcpy(backup_keyfile, keyfile);
     
-    if (change || add || get) {
+    if (change || add || (get && !unique_filename) || delete) {
 	strcat(work_keyfile, ".work");
 	strcat(backup_keyfile, ".old");
 	
 	copy_keyfile(keyfile, backup_keyfile);
     }
     
-    if (add || get)
+    if (add || (get && !unique_filename))
 	copy_keyfile(backup_keyfile, work_keyfile);
 
     keyfile_mode = get_mode(keyfile);
 
-    if (change || list) {
-	if ((backup_keyfile_fd = open(backup_keyfile, O_RDONLY, 0)) < 0) {
-	  fprintf(stderr, "%s: Unable to read %s: %s\n", progname,
-			   backup_keyfile, strerror(errno));
-	    return 1;
-	}
-    }
+    if (change || list || delete)
+	if ((backup_keyfile_fd = open(backup_keyfile, O_RDONLY, 0)) < 0)
+	    err (1, "open %s", backup_keyfile);
 
-    if (change) {
+    if (change || delete) {
 	if ((work_keyfile_fd = 
 	     open(work_keyfile, O_WRONLY | O_CREAT | O_TRUNC, 
-		  SRVTAB_MODE)) < 0) {
-	    fprintf(stderr, "%s: Unable to write %s: %s\n", progname,
-			   work_keyfile, strerror(errno));
-	    return 1;
-	}
+		  SRVTAB_MODE)) < 0)
+	    err (1, "creat %s", work_keyfile);
     }
     else if (add) {
 	if ((work_keyfile_fd =
-	     open(work_keyfile, O_APPEND | O_WRONLY, SRVTAB_MODE)) < 0) {
-	    fprintf(stderr, "%s: Unable to open %s for append: %s\n",
-			   progname, work_keyfile, strerror(errno));
-	    return 1;
-	}
+	     open(work_keyfile, O_APPEND | O_WRONLY, SRVTAB_MODE)) < 0)
+	    err (1, "open with append %s", work_keyfile );
     }
-    else if (get) {
+    else if (get && !unique_filename) {
 	if ((work_keyfile_fd =
-	     open(work_keyfile, O_RDWR | O_CREAT, SRVTAB_MODE)) < 0) {
-	     fprintf(stderr, "%s: Unable to open %s for writing: %s\n",
-			    progname, work_keyfile,
-			    strerror(errno));
-	     return 1;
-	}
+	     open(work_keyfile, O_RDWR | O_CREAT, SRVTAB_MODE)) < 0)
+	    err (1, "open for writing %s", work_keyfile);
     }
     
-    if (change || list) {
+    if (change || list || delete) {
 	while ((getst(backup_keyfile_fd, sname, SNAME_SZ) > 0) &&
 	       (getst(backup_keyfile_fd, sinst, INST_SZ) > 0) &&
 	       (getst(backup_keyfile_fd, srealm, REALM_SZ) > 0) &&
@@ -508,8 +473,9 @@ main(int argc, char **argv)
 		printf("\n");
 	    }
 	    else if (change) {
-		sprintf(change_tkt, TKT_ROOT "_ksrvutil.%d",
-			       (int)getpid());
+		snprintf(change_tkt, sizeof(change_tkt),
+			 TKT_ROOT "_ksrvutil.%u",
+			 (unsigned)getpid());
 		krb_set_tkt_string(change_tkt);
 		destroyp = TRUE;
 
@@ -518,10 +484,8 @@ main(int argc, char **argv)
 		printf("; version %d\n", key_vno);
 		if (interactive)
 		    change_this_key = yn("Change this key?");
-		else if (change)
-		    change_this_key = 1;
 		else
-		    change_this_key = 0;
+		    change_this_key = 1;
 		
 		if (change_this_key)
 		    printf("Changing to version %d.\n", key_vno + 1);
@@ -550,8 +514,8 @@ main(int argc, char **argv)
 			key_vno++;
 		    else {
 		        memcpy(new_key, old_key, sizeof(new_key));
-			fprintf(stderr, "%s: Key NOT changed: %s\n",
-				progname, krb_get_err_text(status));
+			warnx ("Key NOT changed: %s\n",
+			       krb_get_err_text(status));
 			change_this_key = FALSE;
 		    }
 		}
@@ -572,7 +536,7 @@ main(int argc, char **argv)
 			dest_tkt();
 		    }
 		    else {
-			com_err(progname, status, 
+			com_err(__progname, status, 
 				" attempting to change password.");
 			dest_tkt();
 			/* XXX This knows the format of a keyfile */
@@ -584,15 +548,26 @@ main(int argc, char **argv)
 				       old_key, sizeof(des_cblock));
 			    fsync(work_keyfile_fd);
 			    fprintf(stderr,"Key NOT changed.\n");
-			}
-			else {
-			    fprintf(stderr, 
-					  "%s: Unable to revert keyfile: %s\n",
-					  progname, strerror(errno));
+			} else {
+			    warn ("Unable to revert keyfile");
 			    leave("", 1);
 			}
 		    }
 		}
+	    } else if(delete) {
+		int delete_this_key;
+		printf("\nPrincipal: ");
+		print_name(sname, sinst, srealm);
+		printf("; version %d\n", key_vno);
+		delete_this_key = yn("Delete this key?");
+		
+		if (delete_this_key)
+		    printf("Deleting this key.\n");
+		
+		if (!delete_this_key) {
+		    append_srvtab(work_keyfile, work_keyfile_fd, 
+				  sname, sinst, srealm, key_vno, old_key);
+		}
 	    }
 	    memset(old_key, 0, sizeof(des_cblock));
 	    memset(new_key, 0, sizeof(des_cblock));
@@ -604,18 +579,16 @@ main(int argc, char **argv)
 		safe_read_stdin("Name: ", databuf, sizeof(databuf));
 		strncpy(sname, databuf, sizeof(sname) - 1);
 		if (strchr(sname, '.') != 0) {
-		  strcpy(sinst, strchr(sname, '.') + 1);
-		  *(strchr(sname, '.')) = 0;
+		    strcpy(sinst, strchr(sname, '.') + 1);
+		    *(strchr(sname, '.')) = 0;
 		} else {
-		  safe_read_stdin("Instance: ", databuf, sizeof(databuf));
-		  strncpy(sinst, databuf, sizeof(sinst) - 1);
+		    safe_read_stdin("Instance: ", databuf, sizeof(databuf));
+		    strncpy(sinst, databuf, sizeof(sinst) - 1);
 		}
 		safe_read_stdin("Realm: ", databuf, sizeof(databuf));
 		strncpy(srealm, databuf, sizeof(srealm) - 1);
 		safe_read_stdin("Version number: ", databuf, sizeof(databuf));
 		key_vno = atoi(databuf);
-		if (key_vno == 0)
-			key_vno = 1; /* Version numbers are never 0 */
 		if (!srealm[0])
 		    strcpy(srealm, local_realm);
 		printf("New principal: ");
@@ -634,29 +607,19 @@ main(int argc, char **argv)
 	} while (yn("Would you like to add another key?"));
     }
     else if (get) {
-        ksrvutil_get(work_keyfile_fd, work_keyfile,
+        ksrvutil_get(unique_filename, work_keyfile_fd, work_keyfile,
 		     argc - optind, argv + optind);
     }
 
-    if (change || list) 
-	if (close(backup_keyfile_fd) < 0) {
-	    fprintf(stderr, "%s: Failure closing %s: %s\n",
-			   progname, backup_keyfile, strerror(errno));
-	    fprintf(stderr, "continuing...\n");
-	}
+    if (change || list || delete) 
+	if (close(backup_keyfile_fd) < 0)
+	    warn ("close %s", backup_keyfile);
     
-    if (change || add || get) {
-	if (close(work_keyfile_fd) < 0) {
-	    fprintf(stderr, "%s: Failure closing %s: %s\n",
-			   progname, work_keyfile, strerror(errno));
-	    return 1;
-	}
-	if (rename(work_keyfile, keyfile) < 0) {
-	    fprintf(stderr, "%s: Failure renaming %s to %s: %s\n",
-			   progname, work_keyfile, keyfile, 
-			   strerror(errno));
-	    return 1;
-	}
+    if (change || add || (get && !unique_filename) || delete) {
+	if (close(work_keyfile_fd) < 0)
+	    err (1, "close %s", work_keyfile);
+	if (rename(work_keyfile, keyfile) < 0)
+	    err (1, "rename(%s, %s)", work_keyfile, keyfile);
 	chmod(backup_keyfile, keyfile_mode);
 	chmod(keyfile, keyfile_mode);
 	printf("Old keyfile in %s.\n", backup_keyfile);
