@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,7 @@
 
 #include "kadm_locl.h"
 
-RCSID("$Id: new_pwd.c,v 1.8 1996/11/17 21:12:55 assar Exp $");
+RCSID("$Id: new_pwd.c,v 1.12 1998/01/16 23:12:49 joda Exp $");
 
 #ifdef NOENCRYPTION
 #define read_long_pw_string placebo_read_pw_string
@@ -49,24 +49,22 @@ RCSID("$Id: new_pwd.c,v 1.8 1996/11/17 21:12:55 assar Exp $");
 static char *
 check_pw (char *pword)
 {
-    if (strlen(pword) == 0)
-	return "Null passwords are not allowed - Please enter a longer password.";
-    
-    if (strlen(pword) < MIN_KPW_LEN)
+    int ret = kadm_check_pw(pword);
+    switch(ret) {
+    case 0:
+	return NULL;
+    case KADM_PASS_Q_NULL:
+	return "Null passwords are not allowed - "
+	    "Please enter a longer password.";
+    case KADM_PASS_Q_TOOSHORT:
 	return "Password is to short - Please enter a longer password.";
-    
-    /* Don't allow all lower case passwords regardless of length */
-    {
-	char *t;
-	for (t = pword; *t && islower(*t); t++)
-	    ;
-	if (*t == 0)
-	    return "Please don't use an all-lower case password.\n"
-	      "\tUnusual capitalization, delimiter characters or "
-	      "digits are suggested.";
+    case KADM_PASS_Q_CLASS:
+	/* XXX */
+	return "Please don't use an all-lower case password.\n"
+	    "\tUnusual capitalization, delimiter characters or "
+	    "digits are suggested.";
     }
-
-    return NULL;
+    return "Password is insecure"; /* XXX this shouldn't happen */
 }
 
 int
@@ -93,11 +91,11 @@ get_pw_new_pwd(char *pword, int pwlen, krb_principal *pr, int print_realm)
     {
 	char *q;
 	krb_unparse_name_r(pr, p);
-	if(print_realm == 0 && (q == strrchr(p, '@')))
+	if(print_realm == 0 && (q = strrchr(p, '@')))
 	    *q = 0;
     }
 
-    sprintf(ppromp,"Old password for %s:", p);
+    snprintf(ppromp, sizeof(ppromp), "Old password for %s:", p);
     if (read_long_pw_string(pword, pwlen-1, ppromp, 0)) {
 	fprintf(stderr, "Error reading old password.\n");
 	return -1;
@@ -119,7 +117,7 @@ get_pw_new_pwd(char *pword, int pwlen, krb_principal *pr, int print_realm)
     
     do {
 	char verify[MAX_KPW_LEN];
-	sprintf(npromp,"New Password for %s:",p);
+	snprintf(npromp, sizeof(npromp), "New Password for %s:",p);
 	if (read_long_pw_string(pword, pwlen-1, npromp, 0)) {
 	    fprintf(stderr,
 		    "Error reading new password, password unchanged.\n");
@@ -131,7 +129,7 @@ get_pw_new_pwd(char *pword, int pwlen, krb_principal *pr, int print_realm)
 	    continue;
 	}
 	/* Now we got an ok password, verify it. */
-	sprintf(npromp,"Verifying New Password for %s:", p);
+	snprintf(npromp, sizeof(npromp), "Verifying New Password for %s:", p);
 	if (read_long_pw_string(verify, MAX_KPW_LEN-1, npromp, 0)) {
 	    fprintf(stderr,
 		    "Error reading new password, password unchanged.\n");
