@@ -20,7 +20,9 @@
 
 #include <kafs.h>
 
-RCSID("$Id: klist.c,v 1.31 1997/12/08 21:57:08 joda Exp $");
+#include <parse_time.h>
+
+RCSID("$Id: klist.c,v 1.35 1998/05/01 05:16:33 joda Exp $");
 
 static int option_verbose = 0;
 
@@ -36,6 +38,21 @@ short_date(int32_t dp)
     return (cp);
 }
 
+/* prints the approximate kdc time differential as something human
+   readable */
+
+static void
+print_time_diff(void)
+{
+    int d = abs(krb_get_kdc_time_diff());
+    char buf[80];
+
+    if ((option_verbose && d > 0) || d > 60) {
+	unparse_time_approx (d, buf, sizeof(buf));
+	printf ("Time diff:\t%s\n", buf);
+    }
+}
+
 static void
 display_tktfile(char *file, int tgt_test, int long_form)
 {
@@ -105,7 +122,9 @@ display_tktfile(char *file, int tgt_test, int long_form)
      */
        
     if (!tgt_test && long_form)
-	printf("Principal:\t%s\n\n", krb_unparse_name(&pr));
+	printf("Principal:\t%s\n", krb_unparse_name(&pr));
+    print_time_diff();
+    printf("\n");
     while ((k_errno = tf_get_cred(&c)) == KSUCCESS) {
 	if (!tgt_test && long_form && header) {
 	    printf("%-15s  %-15s  %s%s\n",
@@ -125,9 +144,11 @@ display_tktfile(char *file, int tgt_test, int long_form)
 	    continue;			/* not a tgt */
 	}
 	if (long_form) {
+	    struct timeval tv;
 	    strcpy(buf1, short_date(c.issue_date));
 	    c.issue_date = krb_life_to_time(c.issue_date, c.lifetime);
-	    if (time(0) < (unsigned long) c.issue_date)
+	    krb_kdctimeofday(&tv);
+	    if (option_verbose || tv.tv_sec < (unsigned long) c.issue_date)
 	        strcpy(buf2, short_date(c.issue_date));
 	    else
 	        strcpy(buf2, ">>> Expired <<<");
