@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,16 +38,32 @@
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#include "protos.h"
 
-RCSID("$Id: rnd_keys.c,v 1.40 1996/11/15 18:39:07 bg Exp $");
+RCSID("$Id: rnd_keys.c,v 1.53 1998/02/24 14:26:08 bg Exp $");
 #endif
 
 #include <des.h>
 #include <des_locl.h>
+#ifdef KRB5
+#include <krb5-types.h>
+#elif defined(KRB4)
+#include <ktypes.h>
+#endif
 
-#include <sys/bitypes.h>
+#include <string.h>
+
+#ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
+#include <time.h>
+#elif defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -56,9 +72,16 @@ RCSID("$Id: rnd_keys.c,v 1.40 1996/11/15 18:39:07 bg Exp $");
 #include <io.h>
 #endif
 
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
+#endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#include <string.h>
+#endif
+
+#ifdef HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
 
 /*
  * Generate "random" data by checksumming a file.
@@ -149,7 +172,8 @@ static volatile unsigned char *gdata; /* Global data */
 static volatile int igdata;	/* Index into global data */
 static int gsize;
 
-#ifndef WIN32	/* Visual C++ 4.0 (Windows95/NT) */
+#if !defined(WIN32) && !defined(__EMX__) && !defined(__OS2__) && !defined(__CYGWIN32__)
+/* Visual C++ 4.0 (Windows95/NT) */
 
 static
 RETSIGTYPE
@@ -158,6 +182,9 @@ sigALRM(int sig)
     if (igdata < gsize)
 	gdata[igdata++] ^= counter & 0xff;
 
+#ifndef HAVE_SIGACTION
+    signal(SIGALRM, sigALRM); /* Reinstall SysV signal handler */
+#endif
     SIGRETURN(0);
 }
 
@@ -172,9 +199,7 @@ sigALRM(int sig)
 #endif
 #endif
 
-#ifdef HAVE_RANDOM
-
-void
+static void
 des_not_rand_data(unsigned char *data, int size)
 {
   int i;
@@ -185,12 +210,11 @@ des_not_rand_data(unsigned char *data, int size)
     data[i] ^= random() % 0x100;
 }
 
-#endif
-
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__EMX__) && !defined(__OS2__) && !defined(__CYGWIN32__)
 
 #ifndef HAVE_SETITIMER
-void pacemaker(struct timeval *tv)
+static void
+pacemaker(struct timeval *tv)
 {
     fd_set fds;
     pid_t pid;
@@ -213,9 +237,15 @@ void
 des_rand_data(unsigned char *data, int size)
 {
     struct itimerval tv, otv;
+#ifdef HAVE_SIGACTION
     struct sigaction sa, osa;
+#else
+    RETSIGTYPE (*osa)(int);
+#endif
     int i, j;
+#ifndef HAVE_SETITIMER
     pid_t pid;
+#endif
     char *rnd_devices[] = {"/dev/random",
 			   "/dev/srandom",
 			   "/dev/urandom",
@@ -240,11 +270,15 @@ des_rand_data(unsigned char *data, int size)
     gsize = size;
     igdata = 0;
 
+#ifdef HAVE_SIGACTION
     /* Setup signal handler */
     sa.sa_handler = sigALRM;
     sa.sa_flags = 0;
     sigemptyset(&sa.sa_mask);
     sigaction(SIGALRM, &sa, &osa);
+#else
+    osa = signal(SIGALRM, sigALRM);
+#endif
   
     /* Start timer */
     tv.it_value.tv_sec = 0;
@@ -274,13 +308,17 @@ des_rand_data(unsigned char *data, int size)
     kill(pid, SIGKILL);
     waitpid(pid, NULL, 0);
 #endif
+#ifdef HAVE_SIGACTION
     sigaction(SIGALRM, &osa, 0);
+#else
+    signal(SIGALRM, osa != SIG_ERR ? osa : SIG_DFL);
+#endif
 }
 #else
 void
 des_rand_data(unsigned char *p, int s)
 {
-  return des_not_rand_data (p, s);
+  des_not_rand_data (p, s);
 }
 #endif
 
