@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -34,7 +29,7 @@ or implied warranty.
 
 #include "kadm_locl.h"
 
-RCSID("$Id: kadm_cli_wrap.c,v 1.19 1996/11/19 22:48:44 assar Exp $");
+RCSID("$Id: kadm_cli_wrap.c,v 1.22 1997/08/17 07:30:04 assar Exp $");
 
 #ifndef NULL
 #define NULL 0
@@ -129,6 +124,7 @@ kadm_cli_conn(void)
 	signal(SIGPIPE, opipe);
 	return KADM_NO_HERE;		/* couldnt find out who we are */
     }
+#if defined(SO_KEEPALIVE) && defined(HAVE_SETSOCKOPT)
     if (setsockopt(client_parm.admin_fd, SOL_SOCKET, SO_KEEPALIVE,
 		   (void *)&on,
 		   sizeof(on)) < 0) {
@@ -137,6 +133,7 @@ kadm_cli_conn(void)
 	signal(SIGPIPE, opipe);
 	return KADM_NO_CONN;		/* XXX */
     }
+#endif
     return KADM_SUCCESS;
 }
 
@@ -386,6 +383,16 @@ int kadm_change_pw_plain(unsigned char *newkey, char *password, char **pw_msg)
 	return(retc);
 }
 
+/*
+ * This function is here for compatibility with CNS
+ */
+
+int kadm_change_pw2(unsigned char *newkey, char *password, char **pw_msg)
+{
+    return kadm_change_pw_plain (newkey, password, pw_msg);
+}
+
+
 /*
  * kadm_change_pw
  * recieves    : key 
