@@ -43,7 +43,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: KClient.c,v 1.7 1997/04/20 18:11:27 assar Exp $");
+RCSID("$Id: KClient.c,v 1.10 1997/09/27 22:01:54 joda Exp $");
 #endif
 
 #ifdef WIN32	/* Visual C++ 4.0 (Windows95/NT) */
@@ -64,6 +64,32 @@ msg(char *text)
     HWND wnd = GetActiveWindow();
     MessageBox(wnd, text, "KClient message", MB_OK|MB_APPLMODAL);
 }
+ 
+BOOL
+SendTicketForService(LPSTR service, LPSTR version, int fd)
+{
+    KTEXT_ST ticket;
+    MSG_DAT mdat;
+    CREDENTIALS cred;
+    des_key_schedule schedule;
+    char name[SNAME_SZ], inst[INST_SZ], realm[REALM_SZ];
+    int ret;
+    static KClientSessionInfo foo;
+    KClientKey key;
+
+    kname_parse(name, inst, realm, service);
+    strcpy(foo.realm, realm);
+
+    if(KClientStatus(&foo) == KClientNotLoggedIn)
+	KClientLogin(&foo, &key);
+
+    ret = krb_sendauth (0, fd, &ticket,
+			name, inst, realm, 17, &mdat,
+			&cred, &schedule, NULL, NULL, version);
+    if(ret)
+	return FALSE;
+    return TRUE;
+}
 
 BOOL WINAPI
 DllMain(HANDLE hInst, ULONG reason, LPVOID lpReserved)
@@ -96,8 +122,8 @@ DllMain(HANDLE hInst, ULONG reason, LPVOID lpReserved)
 Kerr
 KClientMessage(char *text, Kerr error)
 {
-	msg(text);
-	return error;
+    msg(text);
+    return error;
 }
 
 /* KClientInitSession
@@ -119,10 +145,16 @@ KClientInitSession(KClientSessionInfo *session,
     session->lPort = lPort;
     session->fAddr = fAddr;
     session->fPort = fPort;
-    *(session->user) = '\0';
-    *(session->inst) = '\0';
-    *(session->realm)= '\0';
-    *guser ='\0';
+    if(tf_get_pname(session->user) != KSUCCESS)
+	*(session->user) = '\0';
+    if(tf_get_pinst(session->inst) != KSUCCESS)
+	*(session->inst) = '\0';
+    krb_get_lrealm (session->realm, 1);
+    if(*(session->user))
+	strcpy(guser, session->user);
+    else
+	*guser ='\0';
+    
     return 0;
 }
 
@@ -149,8 +181,9 @@ KClientGetTicketForService(KClientSessionInfo *session,
 	strcpy(session->realm,realm);
     else
 	strcpy(realm,session->realm);
-    if((err = KClientLogin(session, &k)) != KSUCCESS)
-	return err;
+    if(KClientStatus(session) == KClientNotLoggedIn)
+	if((err = KClientLogin(session, &k)) != KSUCCESS)
+	    return err;
 
     if((err = krb_mk_req(&ticket, serv, inst, realm, 0)) != KSUCCESS)
 	return KClientMessage(KClientErrorText(err,0),err);
