@@ -41,7 +41,7 @@
  * Date: June 1996
  */
 
-/* $Id: KClient.h,v 1.6 1997/04/20 18:10:46 assar Exp $ */
+/* $Id: KClient.h,v 1.7 1997/09/27 20:59:50 joda Exp $ */
 
 #ifndef	KCLIENT_H
 #define	KCLIENT_H
@@ -76,7 +76,7 @@ struct _KClientSessionInfo
 };
 typedef struct _KClientSessionInfo KClientSessionInfo;
 
-#ifdef cplusplus
+#ifdef __cplusplus
 extern "C" {
 #endif
 
@@ -158,7 +158,7 @@ Kerr KClientDecrypt(KClientSessionInfo *session,
 /* KClientErrorText */
 char *KClientErrorText(Kerr err, char *text);
 
-#ifdef cplusplus
+#ifdef __cplusplus
 }
 #endif
 
