@@ -21,7 +21,7 @@ or implied warranty.
 
 #include "kdb_locl.h"
 
-RCSID("$Id: krb_dbm.c,v 1.28 1997/12/05 01:28:07 assar Exp $");
+RCSID("$Id: krb_dbm.c,v 1.33 1998/05/28 15:12:01 joda Exp $");
 
 #include <xdbm.h>
 
@@ -98,8 +98,6 @@ static int non_blocking = 0;
  * Utility routine: generate name of database file.
  */
 
-static char *gen_dbsuffix (char *db_name, char *sfx);
-
 static char *
 gen_dbsuffix(char *db_name, char *sfx)
 {
@@ -112,9 +110,6 @@ gen_dbsuffix(char *db_name, char *sfx)
     return dbsuffix;
 }
 
-static void
-decode_princ_key (datum *key, char *name, char *instance);
-
 static void
 decode_princ_key(datum *key, char *name, char *instance)
 {
@@ -124,9 +119,6 @@ decode_princ_key(datum *key, char *name, char *instance)
     instance[INST_SZ - 1] = '\0';
 }
 
-static void
-encode_princ_contents (datum *contents, Principal *principal);
-
 static void
 encode_princ_contents(datum *contents, Principal *principal)
 {
@@ -157,10 +149,7 @@ static int mylock = 0;
 static int inited = 0;
 
 static int
-kerb_dbl_init (void);
-
-static int
-kerb_dbl_init()
+kerb_dbl_init(void)
 {
     if (!inited) {
 	char *filename = gen_dbsuffix (current_db_name, ".ok");
@@ -177,10 +166,7 @@ kerb_dbl_init()
 }
 
 static void
-kerb_dbl_fini (void);
-
-static void
-kerb_dbl_fini()
+kerb_dbl_fini(void)
 {
     close(dblfd);
     dblfd = -1;
@@ -188,9 +174,6 @@ kerb_dbl_fini()
     mylock = 0;
 }
 
-static int
-kerb_dbl_lock (int mode);
-
 static int
 kerb_dbl_lock(int mode)
 {
@@ -206,17 +189,17 @@ kerb_dbl_lock(int mode)
     }
     switch (mode) {
     case KERB_DBL_EXCLUSIVE:
-	flock_mode = K_LOCK_EX;
+	flock_mode = LOCK_EX;
 	break;
     case KERB_DBL_SHARED:
-	flock_mode = K_LOCK_SH;
+	flock_mode = LOCK_SH;
 	break;
     default:
 	fprintf(stderr, "invalid lock mode %d\n", mode);
 	abort();
     }
     if (non_blocking)
-	flock_mode |= K_LOCK_NB;
+	flock_mode |= LOCK_NB;
     
     if (flock(dblfd, flock_mode) < 0) 
 	return errno;
@@ -224,17 +207,15 @@ kerb_dbl_lock(int mode)
     return 0;
 }
 
-static void kerb_dbl_unlock (void);
-
 static void
-kerb_dbl_unlock()
+kerb_dbl_unlock(void)
 {
     if (!mylock) {		/* lock already unlocked */
 	fprintf(stderr, "Kerberos database lock not locked when unlocking.\n");
 	fflush(stderr);
 	exit(1);
     }
-    if (flock(dblfd, K_LOCK_UN) < 0) {
+    if (flock(dblfd, LOCK_UN) < 0) {
 	fprintf(stderr, "Kerberos database lock error. (unlocking)\n");
 	fflush(stderr);
 	perror("flock");
@@ -243,9 +224,6 @@ kerb_dbl_unlock()
     mylock = 0;
 }
 
-int
-kerb_db_set_lockmode (int mode);
-
 int
 kerb_db_set_lockmode(int mode)
 {
@@ -259,10 +237,7 @@ kerb_db_set_lockmode(int mode)
  */
 
 int
-kerb_db_init (void);
-
-int
-kerb_db_init()
+kerb_db_init(void)
 {
     init = 1;
     return (0);
@@ -274,10 +249,7 @@ kerb_db_init()
  */
 
 void
-kerb_db_fini (void);
-
-void
-kerb_db_fini()
+kerb_db_fini(void)
 {
 }
 
@@ -288,9 +260,6 @@ kerb_db_fini()
  * If the alternate database doesn't exist, nothing is changed.
  */
 
-int
-kerb_db_set_name (char *name);
-
 int
 kerb_db_set_name(char *name)
 {
@@ -312,10 +281,7 @@ kerb_db_set_name(char *name)
  */
 
 time_t
-kerb_get_db_age (void);
-
-time_t
-kerb_get_db_age()
+kerb_get_db_age(void)
 {
     struct stat st;
     char *okname;
@@ -340,9 +306,6 @@ kerb_get_db_age()
  * the server (for example, during slave updates).
  */
 
-static time_t
-kerb_start_update (char *db_name);
-
 static time_t
 kerb_start_update(char *db_name)
 {
@@ -357,9 +320,6 @@ kerb_start_update(char *db_name)
     return age;
 }
 
-static int
-kerb_end_update (char *db_name, time_t age);
-
 static int
 kerb_end_update(char *db_name, time_t age)
 {
@@ -395,16 +355,11 @@ kerb_end_update(char *db_name, time_t age)
 }
 
 static time_t
-kerb_start_read (void);
-
-static time_t
-kerb_start_read()
+kerb_start_read(void)
 {
     return kerb_get_db_age();
 }
 
-static int kerb_end_read (time_t age);
-
 static int
 kerb_end_read(time_t age)
 {
@@ -684,7 +639,9 @@ kerb_db_update(long *db, Principal *principal, unsigned int max)
     for (i = 0; i < max; i++) {
 	encode_princ_contents(&contents, principal);
 	encode_princ_key(&key, principal->name, principal->instance);
-	dbm_store((DBM *)db, key, contents, DBM_REPLACE);
+	if(dbm_store((DBM *)db, key, contents, DBM_REPLACE) < 0)
+	    return found; /* XXX some better mechanism to report
+			     failure should exist */
 #ifdef DEBUG
 	if (kerb_debug & 1) {
 	    fprintf(stderr, "\n put %s %s\n",
@@ -702,9 +659,6 @@ kerb_db_update(long *db, Principal *principal, unsigned int max)
  * successfully updated.
  */
 
-int
-kerb_db_put_principal (Principal *principal, unsigned int max);
-
 int
 kerb_db_put_principal(Principal *principal,
 		      unsigned max)
@@ -723,9 +677,6 @@ kerb_db_put_principal(Principal *principal,
     return (found);
 }
 
-void
-kerb_db_get_stat (DB_stat *s);
-
 void
 kerb_db_get_stat(DB_stat *s)
 {
@@ -744,17 +695,11 @@ kerb_db_get_stat(DB_stat *s)
     /* update local copy too */
 }
 
-void
-kerb_db_put_stat (DB_stat *s);
-
 void
 kerb_db_put_stat(DB_stat *s)
 {
 }
 
-void
-delta_stat (DB_stat *a, DB_stat *b, DB_stat *c);
-
 void
 delta_stat(DB_stat *a, DB_stat *b, DB_stat *c)
 {
@@ -781,16 +726,11 @@ delta_stat(DB_stat *a, DB_stat *b, DB_stat *c)
  */
 
 int
-kerb_db_get_dba (char *dba_name, char *dba_inst, Dba *dba, unsigned int max, int *more);
-
-int
-kerb_db_get_dba(char *dba_name, char *dba_inst, Dba *dba,
-		unsigned max,
-		int *more)
-		/* could have wild card */
-		/* could have wild card */
-		/* max number of name structs to return */
-		/* where there more than 'max' tuples? */
+kerb_db_get_dba(char *dba_name,	/* could have wild card */
+		char *dba_inst,	/* could have wild card */
+		Dba *dba,
+		unsigned max,	/* max number of name structs to return */
+		int *more)	/* where there more than 'max' tuples? */
 {
     *more = 0;
     return (0);
