@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -36,7 +31,7 @@ or implied warranty.
 
 #include <kdc.h>
 
-RCSID("$Id: krb_kdb_utils.c,v 1.19 1996/11/19 22:48:47 assar Exp $");
+RCSID("$Id: krb_kdb_utils.c,v 1.23 1997/05/02 14:29:10 assar Exp $");
 
 /* always try /.k for backwards compatibility */
 static char *master_key_files[] = { MKEYFILE, "/.k", NULL };
@@ -54,7 +49,7 @@ k_strerror(int eno)
   static char emsg[128];
 
   if (eno < 0 || eno >= sys_nerr)
-    sprintf(emsg, "Error %d occurred.", eno);
+    snprintf(emsg, sizeof(emsg), "Error %d occurred.", eno);
   else
     return sys_errlist[eno];
 
@@ -153,7 +148,7 @@ int kdb_new_get_new_master_key(des_cblock *key, des_key_schedule schedule,
 }
 
 int kdb_get_master_key(int prompt, des_cblock *master_key, 
-		       struct des_ks_struct *master_key_sched)
+		       des_key_schedule master_key_sched)
 {
   int ask = (prompt == KDB_GET_TWICE);
 #ifndef RANDOM_MKEY
@@ -187,7 +182,9 @@ int kdb_kstash(des_cblock *master_key, char *file)
    was byte order depedent ... */
 
 void
-kdb_encrypt_key (des_cblock (*in), des_cblock (*out), des_cblock (*master_key), struct des_ks_struct *master_key_sched, int e_d_flag)
+kdb_encrypt_key (des_cblock (*in), des_cblock (*out),
+		 des_cblock (*master_key),
+		 des_key_schedule master_key_sched, int e_d_flag)
 {
 
 #ifdef NOENCRYPTION
@@ -204,10 +201,9 @@ kdb_encrypt_key (des_cblock (*in), des_cblock (*out), des_cblock (*master_key),
 /* Returns master key version if successful, otherwise -1 */
 
 long 
-kdb_verify_master_key (des_cblock *master_key, struct des_ks_struct *master_key_sched, FILE *out)
-                           
-                                       
-                 /* setting this to non-null be do output */
+kdb_verify_master_key (des_cblock *master_key,
+		       des_key_schedule master_key_sched,
+		       FILE *out) /* NULL -> no output */
 {
   des_cblock key_from_db;
   Principal principal_data[1];
