@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "kdb_locl.h"
 
-RCSID("$Id: krb_lib.c,v 1.7 1996/11/17 20:23:44 assar Exp $");
+RCSID("$Id: krb_lib.c,v 1.11 1997/05/07 01:36:08 assar Exp $");
 
 #ifdef DEBUG
 extern int debug;
@@ -98,7 +93,8 @@ kerb_delete_principal(char *name, char *inst)
  */
 
 int
-kerb_get_principal(char *name, char *inst, Principal *principal, unsigned int max, int *more)
+kerb_get_principal(char *name, char *inst, Principal *principal,
+		   unsigned int max, int *more)
                  		/* could have wild card */
                  		/* could have wild card */
                          
@@ -168,9 +164,11 @@ kerb_put_principal(Principal *principal, unsigned int n)
     /* and mod date string */
 
     tp = k_localtime(&principal->mod_date);
-    sprintf(principal->mod_date_txt, "%4d-%2d-%2d",
-		   tp->tm_year > 1900 ? tp->tm_year : tp->tm_year + 1900,
-		   tp->tm_mon + 1, tp->tm_mday); /* January is 0, not 1 */
+    snprintf(principal->mod_date_txt,
+	     sizeof(principal->mod_date_txt),
+	     "%4d-%2d-%2d",
+	     tp->tm_year + 1900,
+	     tp->tm_mon + 1, tp->tm_mday); /* January is 0, not 1 */
 #ifdef DEBUG
     if (kerb_debug & 1) {
 	int i;
