@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: create_auth_reply.c,v 1.10 1996/10/27 16:29:32 joda Exp $");
+RCSID("$Id: create_auth_reply.c,v 1.12 1998/05/26 19:57:23 joda Exp $");
 
 /*
  * This routine is called by the Kerberos authentication server
@@ -99,17 +99,12 @@ create_auth_reply(char *pname,	/* Principal's name */
     
     unsigned char *p = pkt->dat;
 
+    if(n != 0)
+	return NULL;
+    
     p += krb_put_int(KRB_PROT_VERSION, p, 1);
     p += krb_put_int(AUTH_MSG_KDC_REPLY, p, 1);
 
-    if(n != 0){
-	/* barf on old code */
-	krb_warning("create_auth_reply: don't give me no krb3 crap!"
-		    " (n == %d)\n", n);
-	return NULL;
-    }
-
-
     p += krb_put_nir(pname, pinst, prealm, p);
 
     p += krb_put_int(time_ws, p, 4);
