@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,9 +21,16 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: debug_decl.c,v 1.5 1996/09/26 20:57:12 assar Exp $");
+RCSID("$Id: debug_decl.c,v 1.8 1998/05/26 15:39:09 joda Exp $");
 
 /* Declare global debugging variables. */
 
 int krb_ap_req_debug = 0;
 int krb_debug = 0;
+int krb_dns_debug = 0;
+
+int
+krb_enable_debug(void)
+{
+    krb_ap_req_debug = krb_debug = krb_dns_debug = 1;
+}
