@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -38,12 +38,19 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: getaddrs.c,v 1.16 1996/11/19 21:56:11 assar Exp $");
+RCSID("$Id: getaddrs.c,v 1.24 1998/04/26 15:10:44 joda Exp $");
 
-#ifdef HAVE_SYS_IOCTL_H
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
 #include <sys/ioctl.h>
 #endif
 #ifdef HAVE_NET_IF_H
+#ifdef __osf__
+struct rtentry;
+struct mbuf;
+#endif
+#ifdef _AIX
+#undef __P /* XXX hack for AIX 4.3 */
+#endif
 #include <net/if.h>
 #endif
 
@@ -58,11 +65,11 @@ RCSID("$Id: getaddrs.c,v 1.16 1996/11/19 21:56:11 assar Exp $");
 int
 k_get_all_addrs (struct in_addr **l)
 {
-#ifndef SIOCGIFCONF
+#if !defined(SIOCGIFCONF) || !defined(SIOCGIFFLAGS) || !defined(SIOCGIFADDR)
      char name[MaxHostNameLen];
      struct hostent *he;
 
-     if (k_gethostname(name, sizeof(name)) < 0)
+     if (gethostname(name, sizeof(name)) < 0)
 	  return -1;
      he = gethostbyname (name);
      if (he == NULL)
@@ -79,6 +86,7 @@ k_get_all_addrs (struct in_addr **l)
      struct ifconf ifconf;
      int num, j;
      char *p;
+     size_t sz;
 
      fd = socket(AF_INET, SOCK_DGRAM, 0);
      if (fd < 0)
@@ -97,33 +105,26 @@ k_get_all_addrs (struct in_addr **l)
 
      j = 0;
      ifreq.ifr_name[0] = '\0';
-     for (p = ifconf.ifc_buf; p < ifconf.ifc_buf + ifconf.ifc_len;) {
+     for (p = ifconf.ifc_buf; p < ifconf.ifc_buf + ifconf.ifc_len; p += sz) {
           struct ifreq *ifr = (struct ifreq *)p;
+	  sz = sizeof(*ifr);
 #ifdef SOCKADDR_HAS_SA_LEN
-	  size_t sz = sizeof(ifr->ifr_name) + ifr->ifr_addr.sa_len;
-#else
-	  size_t sz = sizeof(*ifr);
+	  sz = max(sz, sizeof(ifr->ifr_name) + ifr->ifr_addr.sa_len);
 #endif
+
 	  if(strncmp(ifreq.ifr_name, ifr->ifr_name, sizeof(ifr->ifr_name))) {
-	       if(ioctl(fd, SIOCGIFFLAGS, ifr) < 0) {
-		    close (fd);
-		    free (*l);
-		    return -1;
+	      if(ioctl(fd, SIOCGIFFLAGS, ifr) < 0)
+		  continue;
+	      if (ifr->ifr_flags & IFF_UP) {
+		  if(ioctl(fd, SIOCGIFADDR, ifr) < 0) 
+		      continue;
+		  (*l)[j++] = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr;
 	       }
-	       if (ifr->ifr_flags & IFF_UP) {
-		    if(ioctl(fd, SIOCGIFADDR, ifr) < 0) {
-			 close (fd);
-			 free (*l);
-			 return -1;
-		    }
-		    (*l)[j++] = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr;
-	       }
-	       ifreq = *ifr;
+	      ifreq = *ifr;
 	  }
-	  p = p + sz;
      }
      if (j != num)
-	  *l = realloc (*l, j * sizeof(struct in_addr));
+	 *l = realloc (*l, j * sizeof(struct in_addr));
      close (fd);
      return j;
 #endif /* SIOCGIFCONF */
