@@ -21,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: getrealm.c,v 1.29 1997/12/05 09:05:12 joda Exp $");
+RCSID("$Id: getrealm.c,v 1.31 1998/03/18 13:46:49 bg Exp $");
 
 #define MATCH_SUBDOMAINS        0
 
@@ -94,8 +94,9 @@ static FILE *
 open_krb_realms(void)
 {
     int i;
-    char file[128];
+    char file[MaxPathLen];
     FILE *res;
+
     for(i = 0; krb_get_krbrealms(i, file, sizeof(file)) == 0; i++)
 	if ((res = fopen(file, "r")) != NULL)
 	    return res;
@@ -178,7 +179,7 @@ krb_realmofhost(const char *host)
 	for (cp = ret_realm; *cp; cp++)
 	    *cp = toupper(*cp);
     } else {
-	krb_get_lrealm(ret_realm, 1);
+	strncpy(ret_realm, krb_get_default_realm(), REALM_SZ); /* Wild guess */
     }
     return ret_realm;
 }
