@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: kdc_reply.c,v 1.9 1997/04/15 21:52:14 assar Exp $");
+RCSID("$Id: kdc_reply.c,v 1.10 1998/03/19 15:45:28 joda Exp $");
 
 static int little_endian; /* XXX ugly */
 
@@ -107,11 +107,14 @@ kdc_reply_cipher(KTEXT reply, KTEXT cip)
 
     if(type == AUTH_MSG_ERR_REPLY){
 	u_int32_t code;
+	/* skip these fields */
 	p += strlen((char*)p) + 1; /* name */
 	p += strlen((char*)p) + 1; /* instance */
 	p += strlen((char*)p) + 1; /* realm */
 	p += 4; /* time */
 	p += krb_get_int(p, &code, 4, little_endian);
+	if(code == 0)
+	    code = KFAILURE; /* things will go bad otherwise */
 	return code;
     }
     if(type != AUTH_MSG_KDC_REPLY)
