@@ -1,5 +1,5 @@
 /*
- * $Id: krb.h,v 1.87 1997/12/05 03:31:15 joda Exp $
+ * $Id: krb.h,v 1.93 1998/05/28 15:16:10 joda Exp $
  *
  * Copyright 1987, 1988 by the Massachusetts Institute of Technology. 
  *
@@ -9,6 +9,11 @@
  * Include file for the Kerberos library. 
  */
 
+#if !defined (__STDC__) && !defined(_MSC_VER)
+#define const
+#define signed
+#endif
+
 #include <ktypes.h>
 #include <time.h>
 
@@ -26,13 +31,13 @@
 #endif
 #endif
 
-#ifdef __STDC__
+#if defined (__STDC__) || defined (_MSC_VER)
 #ifndef __P
 #define __P(x) x
 #endif
 #else
 #ifndef __P
-#define __P(x)
+#define __P(x) ()
 #endif
 #endif
 
@@ -57,6 +62,11 @@ extern int krb_ignore_ip_address; /* To turn off IP address comparison */
 extern int krb_no_long_lifetimes; /* To disable AFS compatible lifetimes */
 extern int krbONE;
 #define         HOST_BYTE_ORDER (* (char *) &krbONE)
+/* Debug variables */
+extern int krb_debug;
+extern int krb_ap_req_debug;
+extern int krb_dns_debug;
+
 
 /* Text describing error codes */
 #define		MAX_KRB_ERRORS	256
@@ -319,18 +329,8 @@ struct krb_host {
 #define	KRB_SENDAUTH_VLEN 8	    /* length for version strings */
 
 
-/* File locking */
-#define   K_LOCK_SH   1		/* Shared lock */
-#define   K_LOCK_EX   2		/* Exclusive lock */
-#define   K_LOCK_NB   4		/* Don't block when locking */
-#define   K_LOCK_UN   8		/* Unlock */
-
 extern char *krb4_version;
 
-struct in_addr;
-
-struct sockaddr_in;
-
 typedef int (*key_proc_t) __P((char*, char*, char*, void*, des_cblock*));
 
 typedef int (*decrypt_proc_t) __P((char*, char*, char*, void*, 
