@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: krb_get_in_tkt.c,v 1.23 1997/12/05 00:17:22 joda Exp $");
+RCSID("$Id: krb_get_in_tkt.c,v 1.26 1998/05/26 18:37:09 joda Exp $");
 
 /*
  * decrypt_tkt(): Given user, instance, realm, passwd, key_proc
@@ -124,7 +124,7 @@ krb_mk_as_req(char *user, char *instance, char *realm,
     p += krb_put_int(AUTH_MSG_KDC_REQUEST, p, 1);
 
     p += krb_put_nir(user, instance, realm, p);
-
+    
     gettimeofday(&tv, NULL);
     p += krb_put_int(tv.tv_sec, p, 4);
     p += krb_put_int(life, p, 1);
@@ -166,9 +166,10 @@ krb_decode_as_rep(char *user, char *instance, char *realm,
 	return INTK_ERR;	/* we need a better code here XXX */
 
     now = time(NULL);
-    if (abs((int)(now - cred->issue_date)) > CLOCK_SKEW) {
+    if(krb_get_config_bool("kdc_timesync"))
+	krb_set_kdc_time_diff(cred->issue_date - now);
+    else if (abs((int)(now - cred->issue_date)) > CLOCK_SKEW)
 	return RD_AP_TIME; /* XXX should probably be better code */
-    }
 
     return 0;
 }
