@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: kuserok.c,v 1.21 1997/04/01 08:18:35 joda Exp $");
+RCSID("$Id: kuserok.c,v 1.22 1997/12/08 23:37:47 joda Exp $");
 
 #define OK 0
 #define NOTOK 1
@@ -99,8 +99,7 @@ krb_kuserok(char *name, char *instance, char *realm, char *luser)
        strcmp(instance, "") == 0 &&
        strcmp(realm, lrealm) == 0)
 	return OK;
-    strcpy(file, pwd->pw_dir);
-    strcat(file, "/.klogin");
+    snprintf(file, sizeof(file), "%s/.klogin", pwd->pw_dir);
 
     f = fopen(file, "r");
     if(f == NULL)
@@ -153,4 +152,3 @@ kuserok(AUTH_DAT *auth, char *luser)
 {
     return krb_kuserok(auth->pname, auth->pinst, auth->prealm, luser);
 }
-
