@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_priv.c,v 1.19 1997/12/05 02:01:59 joda Exp $");
+RCSID("$Id: mk_priv.c,v 1.20 1998/02/19 19:13:40 joda Exp $");
 
 /* application include files */
 #include "krb-archaeology.h"
@@ -107,7 +107,7 @@ krb_mk_priv(void *in, void *out, u_int32_t length,
     memcpy(p, in, length);
     p += length;
     
-    gettimeofday(&tv, NULL);
+    krb_kdctimeofday(&tv);
 
     *p++ =tv.tv_usec / 5000;
     
