@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_req.c,v 1.18 1997/12/05 01:55:27 assar Exp $");
+RCSID("$Id: mk_req.c,v 1.19 1998/02/19 19:13:51 joda Exp $");
 
 static int lifetime = 255;	/* But no longer than TGT says. */
 
@@ -56,7 +56,7 @@ build_request(KTEXT req, char *name, char *inst, char *realm,
 
 
     /* Fill in the times on the request id */
-    gettimeofday(&tv, NULL);
+    krb_kdctimeofday(&tv);
 
     *p++ = tv.tv_usec / 5000; /* 5ms */
     
