@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_safe.c,v 1.22 1997/12/05 02:01:52 joda Exp $");
+RCSID("$Id: mk_safe.c,v 1.23 1998/02/19 19:14:02 joda Exp $");
 
 /* application include files */
 #include "krb-archaeology.h"
@@ -99,7 +99,7 @@ krb_mk_safe(void *in, void *out, u_int32_t length, des_cblock *key,
     memcpy(p, in, length);
     p += length;
     
-    gettimeofday(&tv, NULL);
+    krb_kdctimeofday(&tv);
 
     *p++ = tv.tv_usec/5000; /* 5ms */
     
