@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: netread.c,v 1.5 1996/10/05 00:18:02 joda Exp $");
+RCSID("$Id: netread.c,v 1.7 1997/06/19 23:56:44 assar Exp $");
 
 /*
  * krb_net_read() reads from the file descriptor "fd" to the buffer
@@ -45,7 +40,11 @@ krb_net_read (int fd, void *v, size_t len)
     char *buf = v;
 
     do {
+#ifdef WIN32
+	cc = recv(fd, buf, len, 0);
+#else
 	cc = read(fd, buf, len);
+#endif
 	if (cc < 0)
 	    return(cc);		 /* errno is already set */
 	else if (cc == 0) {
