@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: netwrite.c,v 1.6 1996/11/24 05:20:01 joda Exp $");
+RCSID("$Id: netwrite.c,v 1.8 1997/06/19 23:56:25 assar Exp $");
 
 /*
  * krb_net_write() writes "len" bytes from "buf" to the file
@@ -45,7 +40,11 @@ krb_net_write(int fd, const void *v, size_t len)
     const char *buf = (const char*)v;
 
     do {
+#ifdef WIN32
+	cc = send(fd, buf, wrlen, 0);
+#else
 	cc = write(fd, buf, wrlen);
+#endif
 	if (cc < 0)
 	    return(cc);
 	else {
