@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: read_service_key.c,v 1.7 1996/11/17 20:23:46 assar Exp $");
+RCSID("$Id: read_service_key.c,v 1.9 1997/12/05 01:55:54 assar Exp $");
 
 /*
  * The private keys for servers on a given host are stored in a
@@ -101,8 +96,10 @@ read_service_key(char *service,	/* Service Name */
         /* How about instance */
         if (!wcard && strcmp(inst,instance))
             continue;
-        if (wcard)
+        if (wcard) {
             strncpy(instance,inst,INST_SZ);
+	    instance[INST_SZ - 1] = '\0';
+	}
         /* Is this the right realm */
         if (strcmp(rlm,realm)) 
 	    continue;
