@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: recvauth.c,v 1.16 1996/11/14 16:51:20 joda Exp $");
+RCSID("$Id: recvauth.c,v 1.18 1997/07/05 01:35:15 assar Exp $");
 
 /*
  * krb_recvauth() reads (and optionally responds to) a message sent
@@ -122,10 +117,12 @@ krb_recvauth(int32_t options,	/* bit-pattern of options */
     int32_t priv_len;
     u_char tmp_buf[MAX_KTXT_LEN+max(KRB_SENDAUTH_VLEN+1,21)];
 
-    /* read the protocol version number */
-    if (krb_net_read(fd, krb_vers, KRB_SENDAUTH_VLEN) != KRB_SENDAUTH_VLEN)
-	return(errno);
-    krb_vers[KRB_SENDAUTH_VLEN] = '\0';
+    if (!(options & KOPT_IGNORE_PROTOCOL)) {
+	/* read the protocol version number */
+	if (krb_net_read(fd, krb_vers, KRB_SENDAUTH_VLEN) != KRB_SENDAUTH_VLEN)
+	    return(errno);
+	krb_vers[KRB_SENDAUTH_VLEN] = '\0';
+    }
 
     /* read the application version string */
     if (krb_net_read(fd, version, KRB_SENDAUTH_VLEN) != KRB_SENDAUTH_VLEN)
