@@ -22,7 +22,7 @@ or implied warranty.
 #include "krb_locl.h"
 #include <base64.h>
 
-RCSID("$Id: send_to_kdc.c,v 1.49 1997/12/05 01:57:09 assar Exp $");
+RCSID("$Id: send_to_kdc.c,v 1.56 1998/05/31 16:46:16 joda Exp $");
 
 struct host {
     struct sockaddr_in addr;
@@ -174,11 +174,22 @@ static int udp_socket(void)
 
 static int udp_connect(int s, struct sockaddr_in *adr)
 {
+    if(krb_debug) {
+	krb_warning("connecting to %s udp, port %d\n", 
+		    inet_ntoa(adr->sin_addr), 
+		    ntohs(adr->sin_port));
+    }
     return connect(s, (struct sockaddr*)adr, sizeof(*adr));
 }
 
 static int udp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 {
+    if(krb_debug) {
+	krb_warning("sending %d bytes to %s, udp port %d\n", 
+		    pkt->length,
+		    inet_ntoa(adr->sin_addr), 
+		    ntohs(adr->sin_port));
+    }
     return send(s, pkt->dat, pkt->length, 0);
 }
 
@@ -189,12 +200,23 @@ static int tcp_socket(void)
 
 static int tcp_connect(int s, struct sockaddr_in *adr)
 {
+    if(krb_debug) {
+	krb_warning("connecting to %s, tcp port %d\n", 
+		    inet_ntoa(adr->sin_addr), 
+		    ntohs(adr->sin_port));
+    }
     return connect(s, (struct sockaddr*)adr, sizeof(*adr));
 }
 
 static int tcp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 {
     unsigned char len[4];
+    if(krb_debug) {
+	krb_warning("sending %d bytes to %s, tcp port %d\n", 
+		    pkt->length,
+		    inet_ntoa(adr->sin_addr), 
+		    ntohs(adr->sin_port));
+    }
     krb_put_int(pkt->length, len, 4);
     if(send(s, len, sizeof(len), 0) != sizeof(len))
 	return -1;
@@ -203,6 +225,8 @@ static int tcp_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 
 static int udptcp_recv(void *buf, size_t len, KTEXT rpkt)
 {
+    if(krb_debug)
+	krb_warning("recieved %d bytes on udp/tcp socket\n", len);
     memcpy(rpkt->dat, buf, len);
     rpkt->length = len;
     return 0;
@@ -232,13 +256,16 @@ static int url_parse(const char *url, char *host, size_t len, short *port)
 
 static int http_connect(int s, struct sockaddr_in *adr)
 {
-    char *proxy = getenv(PROXY_VAR);
+    const char *proxy = krb_get_config_string(PROXY_VAR);
     char host[MaxHostNameLen + 1];
     short port;
     struct hostent *hp;
     struct sockaddr_in sin;
-    if(proxy == NULL)
+    if(proxy == NULL) {
+	if(krb_debug)
+	    krb_warning("Not using proxy.\n");
 	return tcp_connect(s, adr);
+    }
     if(url_parse(proxy, host, sizeof(host), &port) < 0)
 	return -1;
     hp = gethostbyname(host);
@@ -248,6 +275,10 @@ static int http_connect(int s, struct sockaddr_in *adr)
     sin.sin_family = AF_INET;
     memcpy(&sin.sin_addr, hp->h_addr, sizeof(sin.sin_addr));
     sin.sin_port = htons(port);
+    if(krb_debug) {
+	krb_warning("connecting to proxy on %s (%s) port %d\n", 
+		    host, inet_ntoa(sin.sin_addr), port);
+    }
     return connect(s, (struct sockaddr*)&sin, sizeof(sin));
 }
 
@@ -259,13 +290,26 @@ http_send(int s, struct sockaddr_in* adr, KTEXT pkt)
 
     if(base64_encode(pkt->dat, pkt->length, &str) < 0)
 	return -1;
-    if(getenv(PROXY_VAR)){
+    if(krb_get_config_string(PROXY_VAR)){
+	if(krb_debug) {
+	    krb_warning("sending %d bytes to %s, tcp port %d (via proxy)\n", 
+			pkt->length,
+			inet_ntoa(adr->sin_addr), 
+			ntohs(adr->sin_port));
+	}
 	asprintf(&msg, "GET http://%s:%d/%s HTTP/1.0\r\n\r\n",
 		 inet_ntoa(adr->sin_addr),
 		 ntohs(adr->sin_port),
 		 str);
-    }else
+    }else {
+	if(krb_debug) {
+	    krb_warning("sending %d bytes to %s, http port %d\n", 
+			pkt->length,
+			inet_ntoa(adr->sin_addr), 
+			ntohs(adr->sin_port));
+	}
 	asprintf(&msg, "GET %s HTTP/1.0\r\n\r\n", str);
+    }
     free(str);
 
     if (msg == NULL)
@@ -295,6 +339,8 @@ http_recv(void *buf, size_t len, KTEXT rpkt)
 	return -1;
     }
     p += 4;
+    if(krb_debug)
+	krb_warning("recieved %d bytes on http socket\n", (tmp + len) - p);
     memcpy(rpkt->dat, p, (tmp + len) - p);
     rpkt->length = (tmp + len) - p;
     free(tmp);
@@ -320,7 +366,7 @@ send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr,
 {
     int i;
     int s;
-    unsigned char buf[2048];
+    unsigned char buf[MAX_KTXT_LEN];
     int offset = 0;
     
     for(i = 0; i < sizeof(protos) / sizeof(protos[0]); i++){
@@ -368,7 +414,9 @@ send_recv(KTEXT pkt, KTEXT rpkt, int proto, struct sockaddr_in *adr,
     return TRUE;
 }
 
-#if defined(linux) && defined(HAVE__DNS_GETHOSTBYNAME)
+/* The configuration line "hosts: dns files" in /etc/nsswitch.conf is
+ * rumored to avoid triggering this bug. */
+#if defined(linux) && defined(HAVE__DNS_GETHOSTBYNAME) && 0
 /* Linux libc 5.3 is broken probably somewhere in nsw_hosts.o,
  * for now keep this kludge. */
 static
