@@ -21,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: sendauth.c,v 1.15 1997/04/18 14:11:36 joda Exp $");
+RCSID("$Id: sendauth.c,v 1.16 1997/12/05 01:57:31 assar Exp $");
 
 /*
  * krb_sendauth() transmits a ticket over a file descriptor for a
@@ -130,6 +130,7 @@ krb_sendauth(int32_t options,	/* bit-pattern of options */
 	char tmp[4];
 	u_int32_t len;
 	char inst[INST_SZ];
+	char *i;
 
 	ret = krb_net_read (fd, tmp, 4);
 	if (ret < 0)
@@ -144,9 +145,11 @@ krb_sendauth(int32_t options,	/* bit-pattern of options */
 	    return -1;
 
 	if (options & KOPT_DONT_CANON)
-	    strncpy (inst, instance, sizeof(inst));
+	    i = instance;
 	else
-	    strncpy (inst, krb_get_phost(instance), sizeof(inst));
+	    i = krb_get_phost(instance);
+	strncpy (inst, i, sizeof(inst));
+	inst[sizeof(inst) - 1] = '\0';
 
 	ret = krb_get_cred (service, inst, realm, cred);
 	if (ret != KSUCCESS)
