@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* This defines the Andrew string_to_key function.  It accepts a password
  * string as input and converts its via a one-way encryption algorithm to a DES
  * encryption key.  It is compatible with the original Andrew authentication
@@ -11,7 +6,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: str2key.c,v 1.9 1996/11/19 22:48:48 assar Exp $");
+RCSID("$Id: str2key.c,v 1.10 1997/03/23 03:53:19 joda Exp $");
 
 static void
 mklower(char *s)
