@@ -21,7 +21,7 @@ or implied warranty.
         
 #include "krb_locl.h"
 
-RCSID("$Id: tf_util.c,v 1.26 1997/12/05 01:30:22 assar Exp $");
+RCSID("$Id: tf_util.c,v 1.32 1998/05/28 15:13:31 joda Exp $");
 
 
 #define TOO_BIG -1
@@ -121,6 +121,11 @@ static int tf_read(void *s, int n);
  * TKT_FIL_LCK  - couldn't lock the file, even after a retry
  */
 
+#ifdef _NO_LOCKING
+#undef flock
+#define flock(F, M) 0
+#endif
+
 int
 tf_init(char *tf_name, int rw)
 {
@@ -180,7 +185,7 @@ tf_init(char *tf_name, int rw)
       return TKT_FIL_ACC;
     }
     for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
-      if (flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
+      if (flock(fd, LOCK_EX | LOCK_NB) < 0) {
 	if (krb_debug)
 	  krb_warning("tf_init: retry %d of write lock of `%s'.\n",
 		      i_retry, tf_name);
@@ -204,7 +209,7 @@ tf_init(char *tf_name, int rw)
   }
 
   for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
-    if (flock(fd, K_LOCK_SH | K_LOCK_NB) < 0) {
+    if (flock(fd, LOCK_SH | LOCK_NB) < 0) {
       if (krb_debug)
 	krb_warning("tf_init: retry %d of read lock of `%s'.\n",
 		    i_retry, tf_name);
@@ -255,9 +260,9 @@ tf_create(char *tf_name)
   fd = open(tf_name, O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);
   if (fd < 0)
     return TKT_FIL_ACC;
-  if (flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
+  if (flock(fd, LOCK_EX | LOCK_NB) < 0) {
     sleep(TF_LCK_RETRY);
-    if (flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
+    if (flock(fd, LOCK_EX | LOCK_NB) < 0) {
       close(fd);
       fd = -1;
       return TKT_FIL_LCK;
@@ -372,6 +377,9 @@ tf_put_pinst(char *inst)
  * EOF          - end of file encountered
  */
 
+#define MAGIC_TICKET_NAME "magic"
+#define MAGIC_TICKET_INST "time-diff"
+	  
 int
 tf_get_cred(CREDENTIALS *c)
 {
@@ -383,6 +391,7 @@ tf_get_cred(CREDENTIALS *c)
       krb_warning ("tf_get_cred called before tf_init.\n");
     return TKT_FIL_INI;
   }
+again:
   if ((k_errno = tf_gets(c->service, SNAME_SZ)) < 2)
     switch (k_errno) {
     case TOO_BIG:
@@ -434,6 +443,16 @@ tf_get_cred(CREDENTIALS *c)
       krb_warning ("tf_get_cred: failed tf_read.\n");
     return TKT_FIL_FMT;
   }
+  if(strcmp(c->service, MAGIC_TICKET_NAME) == 0 &&
+     strcmp(c->instance, MAGIC_TICKET_INST) == 0) {
+      /* we found the magic `time diff' ticket; update the kdc time
+         differential, and then get the next ticket */
+      u_int32_t d;
+
+      krb_get_int(c->ticket_st.dat, &d, 4, 0);
+      krb_set_kdc_time_diff(d);
+      goto again;
+  }
   return KSUCCESS;
 }
 
@@ -449,7 +468,7 @@ void
 tf_close(void)
 {
   if (!(fd < 0)) {
-    flock(fd, K_LOCK_UN);
+    flock(fd, LOCK_UN);
     close(fd);
     fd = -1;		/* see declaration of fd above */
   }
@@ -606,7 +625,7 @@ tf_save_cred(char *service,	/* Service name */
 bad:
   return (KFAILURE);
 }
-	  
+
 int
 tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
 {
@@ -620,7 +639,21 @@ tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
 	tf_close();
 	return INTK_ERR;
     }
-
+    
+    if(krb_get_kdc_time_diff() != 0) {
+	/* Add an extra magic ticket containing the time differential
+           to the kdc. The first ticket defines which realm we belong
+           to, but since this ticket gets the same realm as the tgt,
+           this shouldn't be a problem */
+	CREDENTIALS magic;
+	des_cblock s = { 0, 0, 0, 0, 0, 0, 0, 0 };
+	KTEXT_ST t;
+	int d = krb_get_kdc_time_diff();
+	krb_put_int(d, t.dat, 4);
+	t.length = 4;
+	tf_save_cred(MAGIC_TICKET_NAME, MAGIC_TICKET_INST, cred->realm, s, 
+		     cred->lifetime, 0, &t, cred->issue_date);
+    }
     ret = tf_save_cred(cred->service, cred->instance, cred->realm, 
 		       cred->session, cred->lifetime, cred->kvno,
 		       &cred->ticket_st, cred->issue_date);
