@@ -45,7 +45,7 @@
 #include "krb_locl.h"
 #include "ticket_memory.h"
 
-RCSID("$Id: ticket_memory.c,v 1.10 1997/10/02 04:43:34 joda Exp $");
+RCSID("$Id: ticket_memory.c,v 1.11 1998/06/03 02:30:58 joda Exp $");
 
 void msg(char *text, int error);
 
@@ -337,6 +337,7 @@ tf_get_cred(CREDENTIALS *c)
 
     if(!(TktStore =  getTktMem(0)))
 	return KFAILURE;
+    krb_set_kdc_time_diff(TktStore->kdc_diff);
     if((index = nextCredIndex()) == -1)
 	return EOF;
     if(!(cred = TktStore->cred_vec+index))
@@ -404,6 +405,16 @@ tf_save_cred(char *service,	/* Service name */
 }
 
 
+static void
+set_time_diff(time_t diff)
+{
+    tktmem *TktStore = getTktMem(0);
+    if(TktStore == NULL)
+	return;
+    TktStore->kdc_diff = diff;
+}
+
+
 int
 tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
 {
@@ -418,6 +429,8 @@ tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
 	return INTK_ERR;
     }
 
+    set_time_diff(krb_get_kdc_time_diff());
+
     ret = tf_save_cred(cred->service, cred->instance, cred->realm, 
 		       cred->session, cred->lifetime, cred->kvno,
 		       &cred->ticket_st, cred->issue_date);
