@@ -10,7 +10,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: util.c,v 1.4 1996/05/31 21:00:28 assar Exp $");
+RCSID("$Id: util.c,v 1.6 1996/10/05 00:18:34 joda Exp $");
 
 /*
  * Print some of the contents of the given authenticator structure
@@ -35,7 +35,7 @@ ad_print(AUTH_DAT *x)
 #ifdef NOENCRYPTION
     placebo_cblock_print(x->session);
 #else
-    des_cblock_print_file(x->session,stdout);
+    des_cblock_print_file(&x->session,stdout);
 #endif
     /* skip reply for now */
 }
@@ -51,7 +51,7 @@ placebo_cblock_print(x)
     des_cblock x;
 {
     unsigned char *y = (unsigned char *) x;
-    register int i = 0;
+    int i = 0;
 
     printf(" 0x { ");
 
