@@ -9,7 +9,7 @@
 #include "config.h"
 #include "protos.h"
 
-RCSID("$Id: kerberos.c,v 1.75 1997/12/05 04:42:05 assar Exp $");
+RCSID("$Id: kerberos.c,v 1.76 1997/12/15 00:01:40 assar Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -579,8 +579,9 @@ add_port(struct port_spec **ports, int *num_ports, int port, int type)
     return 0;
 }
 
-void make_sockets(char *port_spec, struct in_addr *i_addr, 
-		  struct descr **fds, int *nfds)
+static void
+make_sockets(char *port_spec, struct in_addr *i_addr, 
+	     struct descr **fds, int *nfds)
 {
     int tp;
     struct in_addr *a;
@@ -845,7 +846,7 @@ main(int argc, char **argv)
 }
 
 
-void
+static void
 read_socket(struct descr *n)
 {
     int b;
