@@ -19,7 +19,7 @@ provided "as is" without express or implied warranty.
 
 #include "slav_locl.h"
 
-RCSID("$Id: kprop.c,v 1.30 1997/12/05 01:34:49 assar Exp $");
+RCSID("$Id: kprop.c,v 1.34 1998/05/28 15:14:06 joda Exp $");
 
 #include "kprop.h"
 
@@ -84,12 +84,7 @@ int get_slaves(struct slave_host **psl, char *file, time_t ok_mtime)
 	if (host == NULL) {
 	    warnx ("Ignoring host '%s' in '%s': %s", 
 		   namebuf, file,
-#ifdef HAVE_H_ERRNO
-		   hstrerror(h_errno)
-#else
-		   "unknown error"
-#endif
-		   );
+		   hstrerror(h_errno));
 	    continue;
 	}
 	(*th) = (struct slave_host *) malloc(sizeof(struct slave_host));
@@ -208,12 +203,7 @@ prop_to_slaves(struct slave_host *sl, int fd, char *fslv)
 		    if (gethostname (my_host_name, sizeof(my_host_name)) != 0) {
 			warnx ("gethostname(%s): %s",
 			       my_host_name,
-#ifdef HAVE_H_ERRNO
-			       hstrerror(h_errno)
-#else
-			       "unknown error"
-#endif			       
-			       );
+			       hstrerror(h_errno));
 			close (s);
 			break;	/* next one can't work either! */
 		    }
@@ -468,7 +458,7 @@ main(int argc, char **argv)
 
     if ((fd = open(floc, O_RDONLY)) < 0)
 	err (1, "open(%s)", floc);
-    if (flock(fd, K_LOCK_SH | K_LOCK_NB))
+    if (flock(fd, LOCK_SH | LOCK_NB))
 	err (1, "flock(%s)", floc);
     if (stat(floc, &stbuf))
 	err (1, "stat(%s)", floc);
@@ -494,7 +484,7 @@ main(int argc, char **argv)
 
     if (!prop_to_slaves(slave_host_list, fd, fslv))
 	errx (1, "propagation failed.");
-    if (flock(fd, K_LOCK_UN))
+    if (flock(fd, LOCK_UN))
 	err (1, "flock(%s, LOCK_UN)", floc);
     fprintf(stderr, "\n\n");
     for (sh = slave_host_list; sh; sh = sh->next) {
